@@ -4,6 +4,7 @@ import sys
 import signal
 import logging
 import argparse
+import asyncio
 try:
     import yaml
 except ImportError:
@@ -24,6 +25,7 @@ from .util import resolve_ndsserver
 from ._qt import create_app
 from .scope import NDScope
 from .exceptions import UnknownChannelError
+from . import nds
 
 
 logging.addLevelName(5, 'DATA')
@@ -32,6 +34,7 @@ logging.basicConfig(
     format="%(name)s: %(message)s",
 )
 
+
 ##################################################
 
 
@@ -58,6 +61,19 @@ def parse_window_str(window):
         window = list(map(z, window))
         return tuple([min(window), max(window)])
 
+
+async def find_bad_channels_async(channel_list):
+    async def worker(channel):
+        out = await asyncio.to_thread(nds.find_channels, channel)
+        if not out:
+            return channel
+    tasks = []
+    for channel in channel_list:
+        tasks.append(asyncio.create_task(worker(channel)))
+    await asyncio.gather(*tasks, return_exceptions=True)
+    return set([task.result() for task in tasks if task.result()])
+
+
 ##################################################
 
 
@@ -372,11 +388,9 @@ def main():
     os.environ['NDSSERVER'] = resolve_ndsserver(args.nds)
 
     if args.check:
-        from . import nds
-        bad_channels = set()
-        for channel in _template.template_extract_channels(template):
-            if not nds.find_channels(channel):
-                bad_channels.add(channel)
+        bad_channels = asyncio.run(
+            find_bad_channels_async(_template.template_extract_channels(template))
+        )
         if bad_channels:
             print("The following channels could not be found:", file=sys.stderr)
             for channel in bad_channels:
@@ -393,7 +407,6 @@ def main():
     if not scope.data.channels:
         if args.single_shot:
             parser.error("Must specify channels for single shot mode.")
-        scope.select_channels()
     elif args.single_shot:
         if not t0:
             parser.error("Must specify a time for single shot mode.")
