@@ -1,4 +1,5 @@
 import re
+import fnmatch
 import functools
 
 from qtpy import QtGui, QtWidgets, QtCore
@@ -79,7 +80,11 @@ class AvailableChannelTreeModel(QtCore.QAbstractItemModel):
         self.insert(channel_list)
 
     def insert(self, channel_list):
-        split_re = re.compile('[:\-_]')
+        # HACK: there's something weird about the matching of this
+        # character set.  If the underscore is at the end of the set,
+        # e.g. '[:-_]', then the matches are wrong.  Is this a bug in
+        # python???
+        split_re = re.compile(r'[_:-]')
         for channel in channel_list:
             current = self.root
             slow = is_slow(channel.sample_rate)
@@ -232,12 +237,11 @@ class AvailableChannelTreeSortFilterProxyModel(QtCore.QSortFilterProxyModel):
                 else:
                     return False
 
+
 class AvailableChannelTreeView(QtWidgets.QTreeView):
     def __init__(self, parent):
         super().__init__(parent)
 
-# ------------------------------------------------------------------------------
-
 
 class AvailableChannelTableModel(QtCore.QAbstractTableModel):
     def __init__(self, channel_list, parent=None):
@@ -306,12 +310,17 @@ class AvailableChannelTableModel(QtCore.QAbstractTableModel):
         return len(self.filtered_channel_list)
 
     def update_filter(self):
-        filtered_channel_list = []
-        for channel in self.full_channel_list:
-            if self.search_pattern.upper() in channel.name.upper():
-                if filter_channel(channel, self.show_slow, self.show_fast, self.show_online_only):
-                    filtered_channel_list.append(channel)
-
+        # FIXME: this is fairly slow, how can we speed this up?
+        def _match(channel):
+            return \
+                filter_channel(channel, self.show_slow, self.show_fast, self.show_online_only) \
+                and (
+                    self.search_pattern in channel.name \
+                    or fnmatch.fnmatch(channel.name, self.search_pattern)
+                )
+        filtered_channel_list = [
+            chan for chan in self.full_channel_list if _match(chan)
+        ]
         self.beginResetModel()
         self.filtered_channel_list = filtered_channel_list
         self.endResetModel()
@@ -321,6 +330,121 @@ class AvailableChannelTableView(QtWidgets.QTableView):
     def __init__(self, parent):
         super().__init__(parent)
 
+
+class ChannelListWidget(*load_ui('channel_list.ui')):
+
+    def __init__(self, available_channel_tree_model, available_channel_table_model, parent=None):
+        super().__init__(parent)
+        self.setupUi(self)
+
+        # available channel tree model
+        self.available_channel_tree_model = available_channel_tree_model
+        self.available_channel_tree_proxy_model = AvailableChannelTreeSortFilterProxyModel()
+        self.available_channel_tree_proxy_model.setSourceModel(self.available_channel_tree_model)
+
+        # available channel tree view
+        self.available_channel_tree_view.setModel(self.available_channel_tree_proxy_model)
+        self.available_channel_tree_view.header().setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
+        self.available_channel_tree_view.header().setSectionsMovable(False)
+        self.available_channel_tree_view.setDragEnabled(True)
+        self.available_channel_tree_view.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
+
+        # available channel table model
+        self.available_channel_table_model = available_channel_table_model
+
+        # available channel table view
+        self.available_channel_table_view.setModel(self.available_channel_table_model)
+        self.available_channel_table_view.horizontalHeader().setSectionResizeMode(0, QtWidgets.QHeaderView.Stretch)
+        self.available_channel_table_view.horizontalHeader().setSectionResizeMode(1, QtWidgets.QHeaderView.ResizeToContents)
+        self.available_channel_table_view.horizontalHeader().setHighlightSections(False)
+        self.available_channel_table_view.setDragEnabled(True)
+        self.available_channel_table_view.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
+        self.available_channel_table_view.setWordWrap(False)
+
+        self.all_radio_button.toggled.connect(self.all_radio_button_toggled_slot)
+        self.slow_radio_button.toggled.connect(self.slow_radio_button_toggled_slot)
+        self.fast_radio_button.toggled.connect(self.fast_radio_button_toggled_slot)
+
+        self.online_checkbox.setStyleSheet('color: {0}'.format(COLOR_ONLINE))
+        self.online_checkbox.stateChanged.connect(self.online_checkbox_state_changed_slot)
+
+        self.search_line_edit.textChanged.connect(self.search_line_edit_text_changed_slot)
+        self.search_line_edit.setClearButtonEnabled(True)
+        self.search_line_edit.setFocus()
+
+        # FIXME: this is a substitute (possibly) for logic that should
+        # maybe be based on whether we're talking to an NDS1 or NDS2
+        # server.  for NDS1 everything is online, and there are
+        # testpoints (which are also online only), so a checkbox to
+        # filter on online is superfulous.
+        all_online = functools.reduce(
+            lambda value, chan: value and chan.online,
+            self.available_channel_table_model.full_channel_list,
+            True,
+        )
+        if all_online:
+            self.online_checkbox.hide()
+            # FIXME: this should be based on the presence of test points
+            self.legendLabel.setText(f"<span style='color: {COLOR_TESTPOINT}'>testpoints in blue</span>")
+        else:
+            self.legendLabel.hide()
+
+    def set_server_info(self, server, nchannels, glob=None):
+        text = f"server: <span style='font-weight: bold'>{server}</span>"
+        text += f" [{nchannels} channels]"
+        if glob and glob != '*':
+            text += f" (channel glob: '{glob}')"
+        self.title.setText(text)
+
+    def search_line_edit_text_changed_slot(self, text):
+        self.available_channel_table_model.search_pattern = text
+        self.available_channel_table_model.update_filter()
+
+    def all_radio_button_toggled_slot(self, checked):
+        if checked:
+            self.available_channel_tree_proxy_model.show_slow = True
+            self.available_channel_tree_proxy_model.show_fast = True
+            self.available_channel_tree_proxy_model.invalidateFilter()
+
+            self.available_channel_table_model.show_slow = True
+            self.available_channel_table_model.show_fast = True
+            self.available_channel_table_model.update_filter()
+
+    def slow_radio_button_toggled_slot(self, checked):
+        if checked:
+            self.available_channel_tree_proxy_model.show_slow = True
+            self.available_channel_tree_proxy_model.show_fast = False
+            self.available_channel_tree_proxy_model.invalidateFilter()
+
+            self.available_channel_table_model.show_slow = True
+            self.available_channel_table_model.show_fast = False
+            self.available_channel_table_model.update_filter()
+
+    def fast_radio_button_toggled_slot(self, checked):
+        if checked:
+            self.available_channel_tree_proxy_model.show_slow = False
+            self.available_channel_tree_proxy_model.show_fast = True
+            self.available_channel_tree_proxy_model.invalidateFilter()
+
+            self.available_channel_table_model.show_slow = False
+            self.available_channel_table_model.show_fast = True
+            self.available_channel_table_model.update_filter()
+
+    def online_checkbox_state_changed_slot(self, state):
+        if state == QtCore.Qt.Unchecked:
+            self.available_channel_tree_proxy_model.show_online_only = False
+            self.available_channel_tree_proxy_model.invalidateFilter()
+
+            self.available_channel_table_model.show_online_only = False
+            self.available_channel_table_model.update_filter()
+        elif state == QtCore.Qt.Checked:
+            self.available_channel_tree_proxy_model.show_online_only = True
+            self.available_channel_tree_proxy_model.invalidateFilter()
+
+            self.available_channel_table_model.show_online_only = True
+            self.available_channel_table_model.update_filter()
+
+
 # ------------------------------------------------------------------------------
 
 
@@ -337,7 +461,7 @@ class PushButtonDelegate(QtWidgets.QStyledItemDelegate):
         QtWidgets.QApplication.style().drawControl(QtWidgets.QStyle.CE_PushButton, button, painter)
 
 
-class SelectedChannelTableModel(QtCore.QAbstractTableModel):
+class ConfigChannelTableModel(QtCore.QAbstractTableModel):
     # items in table are NDScopePlotChannel items.  setItemList
     # expects a list of PlotChannel objects, but all other
     # inserts/append operations expect channel name strings.
@@ -345,7 +469,7 @@ class SelectedChannelTableModel(QtCore.QAbstractTableModel):
     def __init__(self, parent=None):
         super().__init__(parent)
         self.item_list = []
-        self.header = ('', 'name', 'color', 'width', 'scale', 'offset', 'unit')
+        self.header = ('', 'name', 'color', 'width', 'scale', 'offset', 'unit', 'label')
 
     def __contains__(self, name):
         """check that channel name already in table or not"""
@@ -380,6 +504,7 @@ class SelectedChannelTableModel(QtCore.QAbstractTableModel):
                 item.params['scale'],
                 item.params['offset'],
                 item.params['unit'],
+                item.params['label'],
             )
             return column_data[index.column()]
         elif role == QtCore.Qt.ForegroundRole:
@@ -397,12 +522,13 @@ class SelectedChannelTableModel(QtCore.QAbstractTableModel):
         try:
             return (
                 (QtCore.Qt.ItemNeverHasChildren),
+                (QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemNeverHasChildren),
                 (QtCore.Qt.ItemNeverHasChildren),
-                (QtCore.Qt.ItemNeverHasChildren),
-                (QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemNeverHasChildren),
-                (QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemNeverHasChildren),
-                (QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemNeverHasChildren),
-                (QtCore.Qt.ItemNeverHasChildren),
+                (QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemNeverHasChildren),
+                (QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemNeverHasChildren),
+                (QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemNeverHasChildren),
+                (QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemNeverHasChildren),
+                (QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemNeverHasChildren),
             )[index.column()]
         except IndexError:
             return QtCore.Qt.NoItemFlags
@@ -496,21 +622,29 @@ class SelectedChannelTableModel(QtCore.QAbstractTableModel):
 
     def setData(self, index, value, role):
         if index.isValid() and role == QtCore.Qt.EditRole:
-            if index.column() == 3:
+            if index.column() == 3:  # width
                 if value > 0:
                     self.item_list[index.row()].set_params(width=value)
                     self.dataChanged.emit(index, index, [role])
                     return True
                 else:
                     return False
-            elif index.column() == 4:
+            elif index.column() == 4:  # scale
                 self.item_list[index.row()].set_params(scale=value)
                 self.dataChanged.emit(index, index, [role])
                 return True
-            elif index.column() == 5:
+            elif index.column() == 5:  # offset
                 self.item_list[index.row()].set_params(offset=value)
                 self.dataChanged.emit(index, index, [role])
                 return True
+            elif index.column() == 6:  # unit
+                self.item_list[index.row()].set_params(unit=value)
+                self.dataChanged.emit(index, index, [role])
+                return True
+            elif index.column() == 7:  # label
+                self.item_list[index.row()].set_params(label=value)
+                self.dataChanged.emit(index, index, [role])
+                return True
             else:
                 return False
         else:
@@ -534,7 +668,7 @@ class SelectedChannelTableModel(QtCore.QAbstractTableModel):
         self.endInsertRows()
 
 
-class SelectedChannelTableView(QtWidgets.QTableView):
+class ConfigChannelTableView(QtWidgets.QTableView):
     def __init__(self, parent):
         super().__init__(parent)
         self.color_dialog = QtWidgets.QColorDialog()
@@ -557,152 +691,106 @@ class SelectedChannelTableView(QtWidgets.QTableView):
         self.color_dialog.colorSelected.disconnect()
         item.set_params(color=color.name())
 
-# ------------------------------------------------------------------------------
 
-
-class ChannelListWidget(*load_ui('channel_list.ui')):
-
-    def __init__(self, available_channel_tree_model, available_channel_table_model, parent=None):
-        super().__init__(parent)
-        self.setupUi(self)
-
-        # available channel tree model
-        self.available_channel_tree_model = available_channel_tree_model
-        self.available_channel_tree_proxy_model = AvailableChannelTreeSortFilterProxyModel()
-        self.available_channel_tree_proxy_model.setSourceModel(self.available_channel_tree_model)
-
-        # available channel tree view
-        self.available_channel_tree_view.setModel(self.available_channel_tree_proxy_model)
-        self.available_channel_tree_view.header().setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
-        self.available_channel_tree_view.header().setSectionsMovable(False)
-        self.available_channel_tree_view.setDragEnabled(True)
-        self.available_channel_tree_view.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
-
-        # available channel table model
-        self.available_channel_table_model = available_channel_table_model
-
-        # available channel table view
-        self.available_channel_table_view.setModel(self.available_channel_table_model)
-        self.available_channel_table_view.horizontalHeader().setSectionResizeMode(0, QtWidgets.QHeaderView.Stretch)
-        self.available_channel_table_view.horizontalHeader().setSectionResizeMode(1, QtWidgets.QHeaderView.ResizeToContents)
-        self.available_channel_table_view.horizontalHeader().setHighlightSections(False)
-        self.available_channel_table_view.setDragEnabled(True)
-        self.available_channel_table_view.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
-        self.available_channel_table_view.setWordWrap(False)
-
-        self.all_radio_button.toggled.connect(self.all_radio_button_toggled_slot)
-        self.slow_radio_button.toggled.connect(self.slow_radio_button_toggled_slot)
-        self.fast_radio_button.toggled.connect(self.fast_radio_button_toggled_slot)
-
-        self.online_checkbox.setStyleSheet('color: {0}'.format(COLOR_ONLINE))
-        self.online_checkbox.stateChanged.connect(self.online_checkbox_state_changed_slot)
-
-        self.search_line_edit.textChanged.connect(self.search_line_edit_text_changed_slot)
-        self.search_line_edit.setClearButtonEnabled(True)
-        self.search_line_edit.setFocus()
-
-        # FIXME: this is a substitute (possibly) for logic that should
-        # maybe be based on whether we're talking to an NDS1 or NDS2
-        # server.  for NDS1 everything is online, and there are
-        # testpoints (which are also online only), so a checkbox to
-        # filter on online is superfulous.
-        all_online = functools.reduce(
-            lambda value, chan: value and chan.online,
-            self.available_channel_table_model.full_channel_list,
-            True,
-        )
-        if all_online:
-            self.online_checkbox.hide()
-            # FIXME: this should be based on the presence of test points
-            self.legendLabel.setText(f"<span style='color: {COLOR_TESTPOINT}'>testpoints in blue</span>")
-        else:
-            self.legendLabel.hide()
-
-    def search_line_edit_text_changed_slot(self, text):
-        self.available_channel_table_model.search_pattern = text
-        self.available_channel_table_model.update_filter()
-
-    def all_radio_button_toggled_slot(self, checked):
-        if checked:
-            self.available_channel_tree_proxy_model.show_slow = True
-            self.available_channel_tree_proxy_model.show_fast = True
-            self.available_channel_tree_proxy_model.invalidateFilter()
-
-            self.available_channel_table_model.show_slow = True
-            self.available_channel_table_model.show_fast = True
-            self.available_channel_table_model.update_filter()
-
-    def slow_radio_button_toggled_slot(self, checked):
-        if checked:
-            self.available_channel_tree_proxy_model.show_slow = True
-            self.available_channel_tree_proxy_model.show_fast = False
-            self.available_channel_tree_proxy_model.invalidateFilter()
-
-            self.available_channel_table_model.show_slow = True
-            self.available_channel_table_model.show_fast = False
-            self.available_channel_table_model.update_filter()
-
-    def fast_radio_button_toggled_slot(self, checked):
-        if checked:
-            self.available_channel_tree_proxy_model.show_slow = False
-            self.available_channel_tree_proxy_model.show_fast = True
-            self.available_channel_tree_proxy_model.invalidateFilter()
-
-            self.available_channel_table_model.show_slow = False
-            self.available_channel_table_model.show_fast = True
-            self.available_channel_table_model.update_filter()
-
-    def online_checkbox_state_changed_slot(self, state):
-        if state == QtCore.Qt.Unchecked:
-            self.available_channel_tree_proxy_model.show_online_only = False
-            self.available_channel_tree_proxy_model.invalidateFilter()
-
-            self.available_channel_table_model.show_online_only = False
-            self.available_channel_table_model.update_filter()
-        elif state == QtCore.Qt.Checked:
-            self.available_channel_tree_proxy_model.show_online_only = True
-            self.available_channel_tree_proxy_model.invalidateFilter()
-
-            self.available_channel_table_model.show_online_only = True
-            self.available_channel_table_model.update_filter()
-
-
-class ChannelSelectWidget(*load_ui('channel_select.ui')):
+class ChannelConfigWidget(*load_ui('channel_config.ui')):
 
     def __init__(self, parent=None):
         super().__init__(parent)
         self.setupUi(self)
 
         # selected channel table model
-        self.selected_channel_table_model = SelectedChannelTableModel()
+        self.channel_table_model = ConfigChannelTableModel()
 
         # selected channel table view
-        self.selected_channel_table_view.setModel(self.selected_channel_table_model)
-        self.selected_channel_table_view.horizontalHeader().setSectionResizeMode(0, QtWidgets.QHeaderView.ResizeToContents)
-        self.selected_channel_table_view.horizontalHeader().setSectionResizeMode(1, QtWidgets.QHeaderView.Stretch)
-        self.selected_channel_table_view.horizontalHeader().setSectionResizeMode(2, QtWidgets.QHeaderView.ResizeToContents)
-        self.selected_channel_table_view.horizontalHeader().setSectionResizeMode(3, QtWidgets.QHeaderView.ResizeToContents)
-        self.selected_channel_table_view.horizontalHeader().setSectionResizeMode(4, QtWidgets.QHeaderView.ResizeToContents)
-        self.selected_channel_table_view.horizontalHeader().setSectionResizeMode(5, QtWidgets.QHeaderView.ResizeToContents)
-        self.selected_channel_table_view.horizontalHeader().setSectionResizeMode(6, QtWidgets.QHeaderView.ResizeToContents)
-        self.selected_channel_table_view.horizontalHeader().setHighlightSections(False)
-        self.selected_channel_table_view.setDragEnabled(True)
-        self.selected_channel_table_view.setAcceptDrops(True)
-        self.selected_channel_table_view.setDefaultDropAction(QtCore.Qt.MoveAction)
-        self.selected_channel_table_view.setDragDropOverwriteMode(False)
-        self.selected_channel_table_view.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
-        self.selected_channel_table_view.setWordWrap(False)
+        self.channel_table_view.setModel(self.channel_table_model)
+        self.channel_table_view.horizontalHeader().setSectionResizeMode(0, QtWidgets.QHeaderView.ResizeToContents)
+        self.channel_table_view.horizontalHeader().setSectionResizeMode(1, QtWidgets.QHeaderView.Stretch)
+        self.channel_table_view.horizontalHeader().setSectionResizeMode(2, QtWidgets.QHeaderView.ResizeToContents)
+        self.channel_table_view.horizontalHeader().setSectionResizeMode(3, QtWidgets.QHeaderView.ResizeToContents)
+        self.channel_table_view.horizontalHeader().setSectionResizeMode(4, QtWidgets.QHeaderView.ResizeToContents)
+        self.channel_table_view.horizontalHeader().setSectionResizeMode(5, QtWidgets.QHeaderView.ResizeToContents)
+        self.channel_table_view.horizontalHeader().setSectionResizeMode(6, QtWidgets.QHeaderView.ResizeToContents)
+        self.channel_table_view.horizontalHeader().setHighlightSections(False)
+        self.channel_table_view.setDragEnabled(True)
+        self.channel_table_view.setAcceptDrops(True)
+        self.channel_table_view.setDefaultDropAction(QtCore.Qt.MoveAction)
+        self.channel_table_view.setDragDropOverwriteMode(False)
+        self.channel_table_view.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
+        self.channel_table_view.setWordWrap(False)
         self.push_button_delegate = PushButtonDelegate()
-        self.selected_channel_table_view.setItemDelegateForColumn(0, self.push_button_delegate)
+        self.channel_table_view.setItemDelegateForColumn(0, self.push_button_delegate)
 
     def set_channel_list(self, channel_list):
-        self.selected_channel_table_model.setItemList(channel_list)
+        self.channel_table_model.setItemList(channel_list)
 
     def add_channel(self, channel):
-        self.selected_channel_table_model.add_channel(channel)
+        self.channel_table_model.add_channel(channel)
 
     def get_channel_list(self):
-        return self.selected_channel_table_model.getItemList()
+        return self.channel_table_model.getItemList()
+
+
+# ------------------------------------------------------------------------------
+
+
+class ChannelListDialog(QtWidgets.QDialog):
+    """modeless dialog to show the channel list
+
+    """
+    def __init__(self, channel_list_widget, parent=None):
+        super().__init__(parent)
+
+        self.layout = QtWidgets.QVBoxLayout()
+        self.setLayout(self.layout)
+
+        self.title = QtWidgets.QLabel("NDS Channel List")
+        self.title.setAlignment(QtCore.Qt.AlignCenter)
+        self.layout.addWidget(self.title)
+
+        self.channel_list_widget = channel_list_widget
+        self.channel_list_widget.setParent(self)
+        self.layout.addWidget(self.channel_list_widget)
+
+        self.foot_layout = QtWidgets.QHBoxLayout()
+        self.layout.addLayout(self.foot_layout)
+
+        self.foot_layout.addWidget(QtWidgets.QLabel("Drag channels into plot to add."))
+        self.buttonBox = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Close)
+        self.buttonBox.buttons()[0].setDefault(False)
+        self.buttonBox.buttons()[0].setAutoDefault(False)
+        self.buttonBox.rejected.connect(self.reject)
+        self.foot_layout.addWidget(self.buttonBox)
+
+
+class PlotChannelConfigDialog(*load_ui('channel_config_dialog.ui')):
+    """modal dialog to configure the channels for an NDScopePlotItem
+
+    """
+    done = Signal('PyQt_PyObject')
+
+    def __init__(self, parent=None):
+        super().__init__(parent)
+        self.setupUi(self)
+
+        self.channel_config_widget = ChannelConfigWidget()
+        self.channel_config_widget.setParent(self)
+        self.verticalLayout.insertWidget(1, self.channel_config_widget)
+
+        self.dialog_button_box.accepted.connect(self._done_accepted)
+        self.dialog_button_box.rejected.connect(self._done_rejected)
+
+    def set_plot(self, plot):
+        self.title.setText(f"Configure channels for plot {plot.loc}")
+        channel_dict = plot.get_channels()
+        self.channel_config_widget.set_channel_list(
+            [NDScopePlotChannel(chan, **params) for chan, params in channel_dict.items()]
+        )
+
+    def _done_accepted(self):
+        self.done.emit(self.channel_config_widget.get_channel_list())
+
+    def _done_rejected(self):
+        self.done.emit(None)
 
 
 class ChannelSelectDialog(*load_ui('channel_select_dialog.ui')):
@@ -715,24 +803,26 @@ class ChannelSelectDialog(*load_ui('channel_select_dialog.ui')):
 
         self.channel_list_widget = channel_list_widget
         self.channel_list_widget.setParent(self)
-        self.panel_layout.insertWidget(0, self.channel_list_widget)
+        self.channel_list_layout.addWidget(self.channel_list_widget)
         self.channel_list_widget.available_channel_tree_view.activated.connect(self.available_channel_tree_view_activated_slot)
         self.channel_list_widget.available_channel_table_view.activated.connect(self.available_channel_table_view_activated_slot)
 
-        self.channel_select_widget = ChannelSelectWidget()
-        self.channel_select_widget.setParent(self)
-        self.panel_layout.insertWidget(1, self.channel_select_widget)
+        self.channel_config_widget = ChannelConfigWidget()
+        self.channel_config_widget.setParent(self)
+        self.channel_config_layout.addWidget(self.channel_config_widget)
 
-        self.infoLabel.setText(f"Douple click or drag channels to select.")
+        self.infoLabel.setText(f"Double click or drag channels to select.")
 
         self.dialog_button_box.accepted.connect(self._done_accepted)
         self.dialog_button_box.rejected.connect(self._done_rejected)
 
-    def setSelectedChannelList(self, channel_dict):
-        self.channel_select_widget.set_channel_list(list(channel_dict.values()))
+    def setSelectedChannels(self, channel_dict):
+        self.channel_config_widget.set_channel_list(
+            [NDScopePlotChannel(chan, **params) for chan, params in channel_dict.items()]
+        )
 
     def getSelectedChannelList(self):
-        return self.channel_select_widget.get_channel_list()
+        return self.channel_config_widget.get_channel_list()
 
     def setTitlePlot(self, plot):
         self.title.setText(f"Select/configure channels for plot {plot}")
@@ -745,14 +835,14 @@ class ChannelSelectDialog(*load_ui('channel_select_dialog.ui')):
             item = source_index.internalPointer()
             if item.is_leaf:
                 channel = item.data
-                self.channel_select_widget.add_channel(channel.name)
+                self.channel_config_widget.add_channel(channel.name)
 
     # Appends the item in the available channel table at the supplied
     # index to the end of the selected channel table.
     def available_channel_table_view_activated_slot(self, index):
         if index.column() == 0:
             channel = self.channel_list_widget.available_channel_table_model.itemFromIndex(index)
-            self.channel_select_widget.add_channel(channel.name)
+            self.channel_config_widget.add_channel(channel.name)
 
     def _done_accepted(self):
         self.done.emit(self.getSelectedChannelList())
@@ -761,6 +851,9 @@ class ChannelSelectDialog(*load_ui('channel_select_dialog.ui')):
         self.done.emit(None)
 
 
+# ------------------------------------------------------------------------------
+
+
 def main():
     import os
     import sys
@@ -804,15 +897,15 @@ def main():
     channel_list = list(sorted(channel_dict.values(), key=lambda c: c.name))
     nchannels = len(channel_list)
     print(f"Channel list received: {nchannels} channels")
-    tree_model = AvailableChannelTreeModel(channel_list)
-    table_model = AvailableChannelTableModel(channel_list)
 
     app = create_app()
-
-    cs = ChannelListWidget(tree_model, table_model)
-    cs.title.setText(f"Channel list for <span style='font-weight: bold'>{server_formatted}</span><br/>channel glob: '{args.glob}' [{nchannels} channels]")
-    cs.show()
-
+    clw = ChannelListWidget(
+        AvailableChannelTreeModel(channel_list),
+        AvailableChannelTableModel(channel_list),
+    )
+    clw.set_server_info(server_formatted, nchannels, glob=args.glob)
+    cld = ChannelListDialog(clw)
+    cld.show()
     sys.exit(app.exec_())
 
 
