@@ -5,7 +5,7 @@ from qtpy.QtCore import Signal
 import pyqtgraph as pg
 
 from . import util
-from .const import LABEL_FILL, COLOR_MODE
+from .const import COLOR_MODE, LABEL_ALPHA
 
 
 class Crosshair(QtCore.QObject):
@@ -29,7 +29,7 @@ class Crosshair(QtCore.QObject):
         )
         self.label = pg.TextItem(
             anchor=(1, 1),
-            fill=LABEL_FILL,
+            fill=(0, 0, 0, LABEL_ALPHA),
         )
         self.active_plot = None
         self.pos = None
@@ -42,7 +42,9 @@ class Crosshair(QtCore.QObject):
     def set_color_mode(self, mode):
         fg = COLOR_MODE[mode]['fg']
         bg = COLOR_MODE[mode]['bg']
-        self.label.fill = bg
+        fill_color = bg.color()
+        fill_color.setAlpha(LABEL_ALPHA)
+        self.label.fill.setColor(fill_color)
         self.label.setColor(fg)
         self.hline.pen.setColor(fg)
         self.vline.pen.setColor(fg)
