@@ -7,7 +7,7 @@ import numpy as np
 from qtpy import QtCore
 from pyqtgraph import exporters
 
-from .template import get_color_by_index
+from .template import get_channel_color
 
 ##################################################
 
@@ -236,8 +236,8 @@ TEMPLATE_EXPORT_FUNCTIONS = {
 
 def matplotlib_plot(ddict, t0=0, window=None):
     import matplotlib.pyplot as plt
-    for i, (chan, data) in enumerate(ddict.items()):
-        color = get_color_by_index(i)
+    for chan, data in ddict.items():
+        color = get_channel_color(chan)
         kwargs = {
             'color': color,
             'label': chan,
