@@ -20,49 +20,61 @@ logger = logging.getLogger('NDS  ')
 
 CHANNEL_TYPE_MASK = nds2.channel.CHANNEL_TYPE_ONLINE | nds2.channel.CHANNEL_TYPE_RAW | nds2.channel.CHANNEL_TYPE_RDS | nds2.channel.CHANNEL_TYPE_TEST_POINT
 
-TREND_CTYPES = [
-    nds2.channel.CHANNEL_TYPE_STREND,
-    nds2.channel.CHANNEL_TYPE_MTREND,
-]
+TREND_CTYPE_MAP = {
+    'raw': nds2.channel.CHANNEL_TYPE_RAW,
+    's-trend': nds2.channel.CHANNEL_TYPE_STREND,
+    'm-trend': nds2.channel.CHANNEL_TYPE_MTREND,
+}
+
+TREND_RATE_MAP = {
+    's-trend': 1,
+    'm-trend': 1/60,
+}
 
 ##########
 
 
-def _parse_channel(channel):
+def _parse_channel_string(channel):
+    """parse channel string into (name, cytpe, mod)
+
+    """
     ct = channel.split(',')
-    name = ct[0]
-    if len(ct) > 1:
-        if ct[1] == 's-trend':
-            ctype = nds2.channel.CHANNEL_TYPE_STREND
-            sample_rate = 1
-        elif ct[1] == 'm-trend':
-            ctype = nds2.channel.CHANNEL_TYPE_MTREND
-            sample_rate = 1/60
-    elif 'MON' in name:
-        ctype = nds2.channel.CHANNEL_TYPE_RAW
-        sample_rate = 16
-    else:
-        ctype = nds2.channel.CHANNEL_TYPE_RAW
-        sample_rate = 2**14
-    return name, ctype, sample_rate
+    try:
+        name, mod = ct[0].split('.')
+    except ValueError:
+        name = ct[0]
+        mod = 'raw'
+    try:
+        ctype = ct[1]
+    except IndexError:
+        ctype = 'raw'
+    return name, ctype, mod
 
 
 class FakeChannel:
-    def __init__(self, name, ctype, sample_rate):
+    def __init__(self, name, ctype, sample_rate, unit):
         self.name = name
-        self.channel_type = ctype
+        self.ctype = ctype
         self.sample_rate = sample_rate
+        self.unit = unit
+
+    @property
+    def channel_type(self):
+        return TREND_CTYPE_MAP[self.ctype]
 
     def Units(self):
-        return ''
+        return self.unit
 
     def DataTypeSize(self):
         return 8
 
+    def __str__(self):
+        return f"<{self.__class__.__name__}: '{self.name}' {self.ctype} {self.sample_rate} {self.unit}>"
+
 
 class FakeBuffer:
-    def __init__(self, name, ctype, sample_rate, seconds, nanoseconds, data):
-        self.channel = FakeChannel(name, ctype, sample_rate)
+    def __init__(self, channel, seconds, nanoseconds, data):
+        self.channel = channel
         assert isinstance(seconds, int)
         assert isinstance(nanoseconds, int)
         self.gps_seconds = seconds
@@ -79,14 +91,31 @@ class FakeBuffer:
 
 
 class FakeChannelSource:
-    def __init__(self, channel):
-        self.channel = channel
+    def __init__(self, name, sample_rate, unit):
+        self.name = name
+        self.sample_rate = sample_rate
+        self.unit = unit
         self.amp = np.random.normal(10, 5, 1)
         self.freq = 2*np.pi * np.random.normal(1, 1/2, 1)
         self.phase = 2*np.pi * np.random.random()
         self.offset = np.random.normal(0, 20, 1)
 
-    def sampler(self, t):
+    def get_channel(self, ctype='raw', mod='raw'):
+        if ctype == 'raw':
+            sample_rate = self.sample_rate
+            name = f"{self.name}"
+        else:
+            sample_rate = TREND_RATE_MAP[ctype]
+            name = f"{self.name}.{mod}"
+        return FakeChannel(
+            name,
+            ctype,
+            sample_rate or self.sample_rate,
+            self.unit
+        )
+
+    def sampler(self, t, mod):
+        # FIXME: use mod appropriately
         # signal
         data = self.amp*np.sin(t * self.freq + self.phase)
         # add offset
@@ -101,17 +130,17 @@ class FakeChannelSource:
             data *= np.nan
         return data
 
-    def gen_buf(self, ctype, sample_rate, start, stride):
+    def gen_buf(self, ctype, mod, start, stride):
+        channel = self.get_channel(ctype, mod)
+        sample_rate = channel.sample_rate
         seconds = int(start)
         nanoseconds = int((start % 1) * 1e9)
         nsamples = int(sample_rate*stride)
         t = np.arange(nsamples)/sample_rate + seconds + nanoseconds*1e-9
-        data = self.sampler(t)
+        data = self.sampler(t, mod)
         data = np.where(t < gpsnow(), data, np.nan)
         return FakeBuffer(
-            self.channel,
-            ctype,
-            sample_rate,
+            channel,
             seconds,
             nanoseconds,
             data,
@@ -119,25 +148,50 @@ class FakeChannelSource:
 
 
 class FakeSource:
-    def __init__(self):
-        self.sources = {}
+    def __init__(self, channels):
+        self.sources = {
+            args[0]: FakeChannelSource(*args) for args in channels
+        }
 
     def gen_bufs(self, channels, start, stride):
         bufs = []
         for chan in channels:
-            name, ctype, sample_rate = _parse_channel(chan)
-            if name not in self.sources:
-                self.sources[name] = FakeChannelSource(name)
-            bufs.append(self.sources[name].gen_buf(
-                ctype,
-                sample_rate,
-                start,
-                stride,
-            ))
+            name, ctype, mod = _parse_channel_string(chan)
+            try:
+                buf = self.sources[name].gen_buf(
+                    ctype,
+                    mod,
+                    start,
+                    stride,
+                )
+            except KeyError:
+                #raise RuntimeError(f"Unknown channel: {chan}")
+                raise
+            bufs.append(buf)
         return bufs
 
 
-FAKE_SOURCE = FakeSource()
+FAKE_SOURCE = FakeSource([
+    ('T1:A-B', 2**14, None),
+    ('T1:B-C', 2**12, 'W'),
+    ('T1:C_D', 2**11, 'V'),
+    ('T1:D-F_G', 2**10, None),
+    ('T1:E-F_G_DAQ', 2**9, None),
+    ('T1:F', 2**4, None),
+    ('A', 2**14, None),
+    ('B', 2**12, 'W'),
+    ('C', 2**11, 'V'),
+    ('D', 2**10, None),
+    ('E', 2**9, None),
+    ('F', 2**4, None),
+])
+
+
+def fake_find_channels(**kwargs):
+    return [
+        source.get_channel()
+        for source in FAKE_SOURCE.sources.values()
+    ]
 
 
 def fake_fetch(channels=None, gps_start=None, gps_stop=None, **kwargs):
@@ -204,10 +258,7 @@ def find_channels(channel_glob=None):
         'channel_type_mask': CHANNEL_TYPE_MASK,
     }
     if os.getenv('NDSSERVER', '').lower() == 'fake':
-        if channel_glob in ['*']:
-            return {}
-        else:
-            return {channel_glob: None}
+        func = fake_find_channels
     else:
         func = nds2.find_channels
     logger.debug("find_channels(**{})".format(kwargs))
@@ -255,7 +306,8 @@ def fetch(channels, start_end):
     return func(**kwargs)
 
 
-def parse_channel(channel):
+def parse_nds_channel(channel):
+    """parse nds.channel object into (name, cytpe, mod)"""
     ctype = nds2.channel.channel_type_to_string(channel.channel_type)
     if ctype in ['s-trend', 'm-trend']:
         # HACK: FIXME: work around a bug in nds2-client around 0.16.3:
@@ -267,7 +319,7 @@ def parse_channel(channel):
     else:
         name = channel.name
         mod = 'raw'
-    return name, mod, ctype
+    return name, ctype, mod
 
 
 def _channels_for_trend(channels, trend):
