@@ -7,7 +7,6 @@ from qtpy import QtCore
 from qtpy.QtGui import QPen
 from qtpy.QtCore import Signal
 import pyqtgraph as pg
-from pyqtgraph import PlotItem, PlotDataItem, FillBetweenItem
 
 from gpstime import gpstime, gpsnow
 
@@ -20,48 +19,39 @@ from . import cursors
 
 logger = logging.getLogger('PLOT')
 
+
 ##################################################
 
-CHANNEL_COLORS = {}
+
+# monkey patch to add a set_color_mode method to the AxisItem class
+
+# Copied from pyqtgraph source version 0.12.3.
+def _updateLabel(self):
+    """Internal method to update the label according to the text"""
+    self.label.setHtml(self.labelString())
+    self._adjustSize()
+    self.picture = None
+    self.update()
+
+pg.AxisItem._updateLabel = _updateLabel
 
 
-def hold_over_nan(y):
-    """hold data over subsequent nans
+def _set_color_mode(self, mode):
+    # Modified from pyqtgraph source version 0.12.3 for AxisItem.setTextPen.
+    # https://pyqtgraph.readthedocs.io/en/pyqtgraph-0.12.3/_modules/pyqtgraph/graphicsItems/AxisItem.html#AxisItem.setTextPen
+    fg = const.COLOR_MODE[mode]['fg']
+    pen = QPen(fg)
+    self.picture = None
+    self._textPen = pen
+    self.labelStyle['color'] = self._textPen.color().name()
+    self._updateLabel()
 
-    This function finds all nans in the input array, and produces an
-    creates an output array that is nan everywhere except where the
-    input array was nan.  Where the input array was nan, the output
-    array will be the value of the input array right before the nan.
-    If the first element of the input array is nan then zero will be
-    used.  Example:
-
-    y   = [nan, nan,   2,   3,   4,   5, nan, nan, nan,   9,  10]
-    out = [  0,   0,   0, nan, nan,   5,   5,   5,   5,   5, nan]
-
-    We use this for indicating "bad" data regions in plots.
-
-    """
-    nani = np.where(np.isnan(y))[0]
-    if nani.size == y.size:
-        return np.zeros_like(y, dtype=float)
-    out = np.empty_like(y, dtype=float)
-    out[:] = np.nan
-    if nani.size == 0:
-        return out
-    ti = np.where(np.diff(nani) > 1)[0]
-    nstart = np.append(nani[0], nani[ti+1])
-    nend = np.append(nani[ti], nani[-1]) + 1
-    for s, e in zip(nstart, nend):
-        if s == 0:
-            v = 0
-        else:
-            v = y[s-1]
-        out[s-1:e+1] = v
-    return out
+pg.AxisItem.set_color_mode = _set_color_mode
 
 
 ##################################################
 
+
 class TimeStringAxis(pg.AxisItem):
     def __init__(self, orientation='bottom', **kwargs):
         super().__init__(orientation, **kwargs)
@@ -115,31 +105,73 @@ class TimeStringAxis(pg.AxisItem):
 ##################################################
 
 
-class NDScopePlot(PlotItem):
-    channel_select_request = Signal('PyQt_PyObject')
-    request_reset = Signal()
+class NDScopeViewBox(pg.ViewBox):
+
+    def __init__(self, data_store, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.data = data_store
+
+    # HACK: we overload this function to pass the mouse event to the
+    # context menu popup, instead of the global point position, so
+    # that the menu can extract the local scene point.  See
+    # NDScopePlotMenu.popup
+    def raiseContextMenu(self, ev):
+        menu = self.getMenu(ev)
+        if menu is not None:
+            self.scene().addParentContextMenus(self, menu, ev)
+            menu.popup(ev)
+
+    def translateBy(self, t=None, x=None, y=None):
+        if self.data.online:
+            x = None
+        super().translateBy(t=t, x=x, y=y)
+
+    def scaleBy(self, s=None, center=None, x=None, y=None):
+        if self.data.online:
+            if center:
+                center = pg.Point(0, center.y())
+        super().scaleBy(s=s, center=center, x=x, y=y)
+
+##################################################
+
+
+class NDScopePlot(pg.PlotItem):
+    channel_config_request = Signal('PyQt_PyObject')
     new_plot_request = Signal('PyQt_PyObject')
     remove_plot_request = Signal('PyQt_PyObject')
     t0_reset = Signal(float)
-    t_cursors_enabled = Signal(bool)
+    t_cursors_enable = Signal(bool)
     t_cursor_moved = Signal('PyQt_PyObject')
     log_mode_toggled = Signal('PyQt_PyObject')
 
-    def __init__(self, data_store, *args, loc=None):
+    def __init__(self, data_store, *args, title=None, loc=None):
         """Initialize NDSCope Plot object
 
         """
         super().__init__(
             *args,
+            viewBox=NDScopeViewBox(data_store),
             axisItems={'bottom': TimeStringAxis()},
         )
 
         self.data = data_store
+        self.title = title
         self.loc = loc
 
         # dict of channel:curve
         self.channels = collections.OrderedDict()
 
+        self.t_cursors = cursors.TCursors(self)
+        self.t_cursors.cursor_moved.connect(self._update_t_cursor)
+        self.y_cursors = cursors.YCursors(self)
+
+        # setup the custom context menu.  should be setup after
+        # cursors are created above.  unclear why it needs to be
+        # implemented as a ViewBox menu instead of as the "ctrlMenu"
+        # for the plot.
+        self.ctrlMenu = None
+        self.getViewBox().menu = NDScopePlotMenu(self)
+
         # plot options
         # use automatic downsampling and clipping to reduce the
         # drawing load
@@ -153,7 +185,7 @@ class NDScopePlot(PlotItem):
         self.hideButtons()
         # add legend
         self.legend = legend.Legend()
-        self.legend.setParentItem(self.vb)
+        self.legend.setParentItem(self.getViewBox())
         self.legend.setVisible(False)
         # show grid lines
         self.showGrid(x=True, y=True, alpha=0.2)
@@ -165,40 +197,23 @@ class NDScopePlot(PlotItem):
         # accept drops
         self.setAcceptDrops(True)
 
-        self.ctrlMenu = None
-        vb = self.getViewBox()
-        vb.menu = NDScopePlotMenu(self)
-
         # If this option is not removed then a default font size will
         # be set in the generated html that will not overridable by
         # setting the font.
         self.titleLabel.opts.pop('size')
 
-        self.t_cursors = cursors.TCursors(self)
-        self.t_cursors.cursor_moved.connect(self._update_t_cursor)
-        self.y_cursors = cursors.YCursors(self)
-
     def dropEvent(self, event):
         data = event.mimeData()
         if data.hasFormat('text/plain'):
             text_list = data.text().splitlines()
             for text in text_list:
-                self.add_channel(text)
+                self.add_channels({text: None})
 
-    def add_channel(self, channel, **kwargs):
-        """Add channel to plot
-
-        keyword arguments are passed directly to NDScopePlotChannel.
-
-        """
-        if channel in self.channels:
-            return
-        cc = NDScopePlotChannel(channel, **kwargs)
-        self._add_channel_obj(cc)
+    def open_channel_config_dialog(self):
+        self.channel_config_request.emit(self)
 
     def _add_channel_obj(self, cc):
         channel = cc.channel
-        self.data.add_channel(channel)
         self.channels[channel] = cc
         self.addItem(cc.curves['y'])
         self.addItem(cc.curves['bad'])
@@ -206,28 +221,12 @@ class NDScopePlot(PlotItem):
         self._update_title_legend()
         cc.label_changed.connect(self._update_legend_item)
         cc.label_changed.connect(self._update_title_legend)
+        cc.unit_changed.connect(self._update_units)
         self._update_legend_item(channel)
 
-    def _select_channel_menu(self):
-        self.channel_select_request.emit(self)
-
-    def _add_channel_menu(self, channel):
-        self.add_channel(channel)
-        self.request_reset.emit()
-
-    def remove_channel(self, channel):
-        """remove channel from plot
-
-        """
-        if channel not in self.channels:
-            return
-        cc = self.channels[channel]
-        self._remove_channel_obj(cc)
-
     def _remove_channel_obj(self, cc):
         channel = cc.channel
-        self.data.remove_channel(channel)
-        self.legend.removeItem(cc.label)
+        self.legend.removeItem(cc.get_label())
         for curve in cc.curves.values():
             self.removeItem(curve)
         del self.channels[channel]
@@ -240,12 +239,63 @@ class NDScopePlot(PlotItem):
         be removed.
 
         """
-        set_chans = set(chan_obj_list)
-        cur_chans = set(self.channels.values())
-        for chan in cur_chans - set_chans:
-            self._remove_channel_obj(chan)
-        for chan in set_chans - cur_chans:
-            self._add_channel_obj(chan)
+        channel_dict = {cc.channel: cc for cc in chan_obj_list}
+        set_chans = set(channel_dict.keys())
+        cur_chans = set(self.channels.keys())
+        with self.data as ds:
+            for chan in cur_chans - set_chans:
+                ds.remove_channel(chan)
+                cc = self.channels[chan]
+                self._remove_channel_obj(cc)
+            for chan in set_chans - cur_chans:
+                ds.add_channel(chan)
+                self._add_channel_obj(channel_dict[chan])
+            for chan in set_chans & cur_chans:
+                self.channels[chan].set_params(**channel_dict[chan].params)
+        self.set_y_range('auto')
+
+    def add_channels(self, channel_dict):
+        """Add channels to plot
+
+        Takes a dictionary where the keys are channel names and the
+        values are channel curve keyword argument dictionaries.  The
+        keyword arguments are passed directly to NDScopePlotChannel.
+
+        """
+        with self.data as ds:
+            for channel, kwargs in channel_dict.items():
+                if channel in self.channels:
+                    continue
+                kwargs = kwargs or {}
+                cc = NDScopePlotChannel(channel, **kwargs)
+                ds.add_channel(channel)
+                self._add_channel_obj(cc)
+        self.set_y_range('auto')
+
+    def remove_channels(self, channel_list=None):
+        """remove channels from plot
+
+        Takes a list of channel names to remove.  If the
+        `channel_list` argument is missing all channels will be
+        removed.
+
+        """
+        if channel_list is None:
+            channel_list = list(self.channels.keys())
+        with self.data as ds:
+            for channel in channel_list:
+                if channel not in self.channels:
+                    continue
+                ds.remove_channel(channel)
+                cc = self.channels[channel]
+                self._remove_channel_obj(cc)
+        self.set_y_range('auto')
+
+    def get_channels(self):
+        """get a channel:params dict for all channels in plot
+
+        """
+        return {chan:cc.params for chan, cc in self.channels.items()}
 
     def set_font(self, font):
         """Set label and axis font"""
@@ -310,7 +360,7 @@ class NDScopePlot(PlotItem):
         range should be tuple.
 
         """
-        logger.debug(f"set Y range: {y_range}")
+        logger.debug(f"plot {self.loc} Y range: {y_range}")
         if y_range in [None, 'auto']:
             self.enableAutoRange(axis='y')
         else:
@@ -347,28 +397,25 @@ class NDScopePlot(PlotItem):
 
     def enable_t_cursors(self):
         """enable T cursors and return cursor object"""
-        if self.t_cursors.T1 not in self.getViewBox().allChildren():
-            logger.debug("T cursor enable")
-            self.addItem(self.t_cursors.T1, ignoreBounds=True)
-            self.addItem(self.t_cursors.T2, ignoreBounds=True)
+        if self.t_cursors.C1 not in self.getViewBox().allChildren():
+            self.addItem(self.t_cursors.C1, ignoreBounds=True)
+            self.addItem(self.t_cursors.C2, ignoreBounds=True)
             self.addItem(self.t_cursors.diff, ignoreBounds=True)
             self.t_cursors.reset()
+            logger.debug(f"plot {self.loc} T cursor enabled")
         return self.t_cursors
 
-    def _update_t_cursors(self, enable):
-        self.t_cursors_enabled.emit(enable)
-
     def _update_t_cursor(self, indval):
         self.t_cursor_moved.emit(indval)
 
     def enable_y_cursors(self):
         """enable Y cursors and return cursor object"""
-        if self.y_cursors.Y1 not in self.getViewBox().allChildren():
-            logger.debug("Y cursor enable")
-            self.addItem(self.y_cursors.Y1, ignoreBounds=True)
-            self.addItem(self.y_cursors.Y2, ignoreBounds=True)
+        if self.y_cursors.C1 not in self.getViewBox().allChildren():
+            self.addItem(self.y_cursors.C1, ignoreBounds=True)
+            self.addItem(self.y_cursors.C2, ignoreBounds=True)
             self.addItem(self.y_cursors.diff, ignoreBounds=True)
             self.y_cursors.reset()
+            logger.debug(f"plot {self.loc} Y cursor enabled")
         return self.y_cursors
 
     ##########
@@ -377,20 +424,29 @@ class NDScopePlot(PlotItem):
     def _update_legend_item(self, channel):
         cc = self.channels[channel]
         self.legend.removeItem(cc.curves['y'])
-        self.legend.addItem(cc.curves['y'], cc.label)
+        self.legend.addItem(cc.curves['y'], cc.get_label())
 
     def _update_title_legend(self):
         """update plot title and legend"""
-        if len(self.channels) < 1:
+        if self.title:
+            self.legend.setVisible(True)
+            self.setTitle(self.title)
+        elif len(self.channels) < 1:
             self.legend.setVisible(False)
             self.setTitle(None)
         elif len(self.channels) == 1:
             self.legend.setVisible(False)
-            self.setTitle(list(self.channels.values())[0].label)
+            self.setTitle(list(self.channels.values())[0].get_label())
         else:
             self.legend.setVisible(True)
             self.setTitle(None)
 
+    def _update_units(self):
+        units = set(
+            [cc.get_unit() for cc in self.channels.values()]
+        )
+        self.setLabel('left', '/'.join(list(units)))
+
     def clear_data(self):
         """clear data for all channels
 
@@ -427,10 +483,6 @@ class NDScopePlot(PlotItem):
         self.axes['bottom']['item'].t0 = t0
         self._set_t_limits(t0)
 
-        ctype_changed = False
-
-        units = set()
-
         for channel, cc in self.channels.items():
             if channel not in data or not data[channel]:
                 continue
@@ -448,23 +500,15 @@ class NDScopePlot(PlotItem):
                 if cc.curves['fill']:
                     self.removeItem(cc.curves['fill'])
 
-            ctype_changed |= cc.set_ctype(cd.ctype)
-
             cc.set_data(cd, t0)
 
-            units.add(cc.get_unit(cd.unit))
-
-        self.setLabel('left', '/'.join(list(units)))
-
-        if ctype_changed:
-            self._update_title_legend()
-
 
 ##################################################
 
 
 class NDScopePlotChannel(QtCore.QObject):
     label_changed = Signal(str)
+    unit_changed = Signal(str)
 
     def __init__(self, channel, **kwargs):
         """Initialize channel curve object
@@ -481,47 +525,37 @@ class NDScopePlotChannel(QtCore.QObject):
         """
         super().__init__()
 
-        global CHANNEL_COLORS
-
         self.channel = channel
-        self.ctype = None
-        self.is_trend = None
         self.params = dict(template.CURVE)
-        self.label = channel
+        self.data = None
 
         self.curves = {}
-        self.curves['y'] = PlotDataItem([0, 0], name=self.label)
-        self.curves['min'] = PlotDataItem([0, 0])
-        self.curves['max'] = PlotDataItem([0, 0])
+        self.curves['y'] = pg.PlotDataItem([0, 0], name='')
+        self.curves['min'] = pg.PlotDataItem([0, 0])
+        self.curves['max'] = pg.PlotDataItem([0, 0])
         # FIXME: fill is expensive, so we disable it until figure it out
         if False:
-            self.curves['fill'] = FillBetweenItem(
+            self.curves['fill'] = pg.FillBetweenItem(
                 self.curves['min'],
                 self.curves['max'],
             )
         else:
             self.curves['fill'] = None
         # special curve for bad data (nans, infs)
-        self.curves['bad'] = PlotDataItem(
-            [0, 0],
-            pen=pg.mkPen('r', width=1, style=QtCore.Qt.DotLine)
-        )
-
-        if channel in CHANNEL_COLORS:
-            color = CHANNEL_COLORS[channel]
-        else:
-            while True:
-                color = template.get_pen_color()
-                if color not in CHANNEL_COLORS.values():
-                    break
-        CHANNEL_COLORS[channel] = color
-        self.params['color'] = color
+        self.curves['bad'] = pg.PlotDataItem([0, 0])
 
+        kwargs['color'] = template.get_channel_color(channel, kwargs.get('color'))
         self.set_params(**kwargs)
 
     def as_tuple(self):
         return (('channel', self.channel),) + tuple(sorted(self.params.items()))
 
+    @property
+    def is_trend(self):
+        if self.data:
+            return self.data.is_trend
+        return False
+
     def __repr__(self):
         return "<{} {} {}>".format(
             self.__class__.__name__,
@@ -549,29 +583,27 @@ class NDScopePlotChannel(QtCore.QObject):
         self.transform = transform
 
     def _update_label(self):
-        label = self.channel
-        if self.params['offset'] != 0:
-            label += ' {:+g}'.format(self.params['offset'])
-        if self.params['scale'] != 1:
-            label += ' *{:g}'.format(self.params['scale'])
-        if self.is_trend:
-            label += ' [{}]'.format(self.ctype)
-        self.label = label
         self.label_changed.emit(self.channel)
 
+    def _update_unit(self):
+        self.unit_changed.emit(self.channel)
+
+    def get_QColor(self):
+        """Get channel color as a QColor object"""
+        return pg.mkColor(self.params['color'])
+
     def set_params(self, **params):
         """set parameters for this channel
 
         """
-        PARAMS = ['color', 'width', 'unit', 'scale', 'offset']
+        PARAMS = ['color', 'width', 'scale', 'offset', 'unit', 'label']
 
         for param, value in params.items():
             if param not in PARAMS:
                 raise KeyError(f"invalid parameter '{param}', options are: {PARAMS}")
             if param == 'color':
-                self.params[param] = pg.mkColor(value).name()
-            else:
-                self.params[param] = value
+                value = pg.mkColor(value).name()
+            self.params[param] = value
 
         color = self.get_QColor()
         pen = pg.mkPen(color, width=self.params['width'])
@@ -581,11 +613,43 @@ class NDScopePlotChannel(QtCore.QObject):
         self.curves['y'].setPen(pen)
         self.curves['min'].setPen(mmpen)
         self.curves['max'].setPen(mmpen)
+        self.curves['bad'].setPen(
+            pg.mkPen('r', width=self.params['width'], style=QtCore.Qt.DotLine)
+        )
         if self.curves['fill']:
             self.curves['fill'].setBrush(mmc)
-        self._update_transform()
         self._update_label()
-        self.redraw_data()
+        self._update_unit()
+        if not set(['scale', 'offset']).isdisjoint(self.params):
+            self._update_transform()
+            self._redraw_curves()
+
+    def get_unit(self):
+        """return channel unit"""
+        unit = self.params.get('unit')
+        if not unit:
+            if self.data:
+                unit = self.data.unit or 'counts'
+            else:
+                unit = 'counts'
+        return unit
+
+    def get_label(self):
+        if self.params.get('label'):
+            return self.params['label']
+        offset = self.params['offset']
+        scale = self.params['scale']
+        label = f'{self.channel}'
+        if offset != 0:
+            if scale != 1:
+                label = f'({self.channel} {offset:+g})'
+            else:
+                label += f' {offset:+g}'
+        if scale != 1:
+            label += f'*{scale:g}'
+        if self.data and self.data.is_trend:
+            label += f' [{self.data.ctype}]'
+        return label
 
     def set_mean_curve_visibility(self, visible):
         self.curves['y'].setVisible(visible)
@@ -596,44 +660,21 @@ class NDScopePlotChannel(QtCore.QObject):
     def set_max_curve_visibility(self, visible):
         self.curves['max'].setVisible(visible)
 
-    def get_QColor(self):
-        """Get channel color as a QColor object"""
-        return pg.mkColor(self.params['color'])
-
-    def set_ctype(self, ctype):
-        """Update the channel ctype
-
-        Returns True if type changed, False otherwise
-
-        """
-        if ctype == self.ctype:
-            return False
-        self.ctype = ctype
-        if ctype in ['s-trend', 'm-trend']:
-            self.is_trend = True
-        else:
-            self.is_trend = False
-        self._update_label()
-        return True
-
-    def get_unit(self, default):
-        """return channel unit"""
-        unit = self.params.get('unit')
-        if not unit:
-            unit = default or 'counts'
-        return unit
-
     def clear_data(self):
         """clear data for curves
 
         """
+        self.data = None
         for curve in self.curves.values():
             try:
                 curve.setData(np.array([0, 0]))
             except:
                 pass
 
-    def _set_curve_data(self, mod, t, y):
+    def _update_curve_data(self, mod, cid, t=None):
+        y = self.data[mod]
+        if t is None:
+            t = self.data.tarray - self.t0
         # FIXME: HACK: replace all +-infs with nans.  the infs
         # were causing the following exception in PlotCurveItem
         # when it tried to find the min/max of the array:
@@ -648,12 +689,24 @@ class NDScopePlotChannel(QtCore.QObject):
             # just pass
             pass
         y = self.transform(y)
-        self.curves[mod].setData(
+        self.curves[cid].setData(
             x=t,
             y=y,
             connect="finite",
         )
-        return y
+        return t, y
+
+    def _redraw_curves(self):
+        """redraw all data"""
+        if not self.data:
+            return
+        if 'raw' in self.data:
+            t, y = self._update_curve_data('raw', 'y')
+        else:
+            t, y = self._update_curve_data('mean', 'y')
+            for mod in ['min', 'max']:
+                self._update_curve_data(mod, mod, t)
+        return t, y
 
     def set_data(self, data, t0):
         """set data for curves
@@ -661,14 +714,14 @@ class NDScopePlotChannel(QtCore.QObject):
         Data should be DataBuffer object.
 
         """
-        t = data.tarray - t0
-
-        if 'raw' in data:
-            y = self._set_curve_data('y', t, data['raw'])
+        if self.data:
+            ctype_changed = data.ctype != self.data.ctype
         else:
-            y = self._set_curve_data('y', t, data['mean'])
-            for mod in ['min', 'max']:
-                self._set_curve_data(mod, t, data[mod])
+            ctype_changed = True
+        self.data = data
+        self.t0 = t0
+
+        t, y = self._redraw_curves()
 
         self.curves['bad'].setData(
             x=t,
@@ -676,37 +729,43 @@ class NDScopePlotChannel(QtCore.QObject):
             connect="finite",
         )
 
-    def redraw_data(self):
-        """redraw all data, in case transform has changed"""
-        for mod, curve in self.curves.items():
-            if not curve:
-                continue
-            (t, y) = curve.getData()
-            self._set_curve_data(mod, t, y)
+        if ctype_changed:
+            self._update_label()
+
 
 ##################################################
 
-# This is a monkey patch to add a set_color_mode method to the
-# pg.AxisItem class.
 
-# Copied from pyqtgraph source version 0.12.3.
-def _updateLabel(self):
-    """Internal method to update the label according to the text"""
-    self.label.setHtml(self.labelString())
-    self._adjustSize()
-    self.picture = None
-    self.update()
+def hold_over_nan(y):
+    """hold data over subsequent nans
 
-pg.AxisItem._updateLabel = _updateLabel
+    This function finds all nans in the input array, and produces an
+    output array that is nan everywhere except where the input array
+    was nan.  Where the input array was nan, the output array will be
+    the value of the input array right before the nan.  If the first
+    element of the input array is nan then zero will be used.
+    Example:
 
-def _set_color_mode(self, mode):
-    # Modified from pyqtgraph source version 0.12.3 for AxisItem.setTextPen.
-    # https://pyqtgraph.readthedocs.io/en/pyqtgraph-0.12.3/_modules/pyqtgraph/graphicsItems/AxisItem.html#AxisItem.setTextPen
-    fg = const.COLOR_MODE[mode]['fg']
-    pen = QPen(fg)
-    self.picture = None
-    self._textPen = pen
-    self.labelStyle['color'] = self._textPen.color().name()
-    self._updateLabel()
+    y   = [nan, nan,   2,   3,   4,   5, nan, nan, nan,   9,  10]
+    out = [  0,   0,   0, nan, nan,   5,   5,   5,   5,   5, nan]
 
-pg.AxisItem.set_color_mode = _set_color_mode
+    We use this for indicating "bad" data regions in plots.
+
+    """
+    nani = np.where(np.isnan(y))[0]
+    if nani.size == y.size:
+        return np.zeros_like(y, dtype=float)
+    out = np.empty_like(y, dtype=float)
+    out[:] = np.nan
+    if nani.size == 0:
+        return out
+    ti = np.where(np.diff(nani) > 1)[0]
+    nstart = np.append(nani[0], nani[ti+1])
+    nend = np.append(nani[ti], nani[-1]) + 1
+    for s, e in zip(nstart, nend):
+        if s == 0:
+            v = 0
+        else:
+            v = y[s-1]
+        out[s-1:e+1] = v
+    return out
