@@ -61,6 +61,9 @@ class CursorWidget(QtWidgets.QWidget):
         self._c1.setToolTip(f"enable {check1} cursor")
         self._c2 = QtWidgets.QCheckBox(check2)
         self._c2.setToolTip(f"enable {check2} cursor")
+        self.labels = QtWidgets.QCheckBox("labels")
+        self.labels.setToolTip("show cursor labels")
+        self.labels.setChecked(True)
         setattr(self, check1, self._c1)
         setattr(self, check2, self._c2)
         self.reset = QtWidgets.QPushButton("reset")
@@ -68,14 +71,17 @@ class CursorWidget(QtWidgets.QWidget):
         self.layout = QtWidgets.QHBoxLayout()
         self.layout.addWidget(self._c1)
         self.layout.addWidget(self._c2)
+        self.layout.addWidget(self.labels)
         self.layout.addWidget(self.reset)
+        self.layout.setContentsMargins(0, 5, 0, 5)
         self.setLayout(self.layout)
 
 
 # this is lifted from the pqtgraph.ViewBoxMenu module
 class NDScopePlotMenu(QtWidgets.QMenu):
     def __init__(self, plot):
-        QtWidgets.QMenu.__init__(self)
+        super().__init__()
+
         # keep weakref to view to avoid circular reference (don't know
         # why, but this prevents the ViewBox from being collected)
         self.plot = weakref.ref(plot)
@@ -85,7 +91,7 @@ class NDScopePlotMenu(QtWidgets.QMenu):
         loc = self.plot().loc
         title = f"plot {loc}"
         self.setTitle(title)
-        self.addLabel(title)
+        self.titleLabel = self.addLabel(title)
         self.addSeparator()
 
         self.viewAll = QtWidgets.QAction("view all data", self)
@@ -111,71 +117,81 @@ class NDScopePlotMenu(QtWidgets.QMenu):
         self.addMenu(self.mouseModeUI)
 
         self.addLabel()
-
         self.addSection("T cursors")
+
         self.t_cursor_widget = CursorWidget('T1', 'T2')
-        self.t_cursor_action = QtWidgets.QWidgetAction(self)
-        self.t_cursor_action.setDefaultWidget(self.t_cursor_widget)
         self.t_cursor_widget.T1.stateChanged.connect(self.update_t1_cursor)
         self.t_cursor_widget.T2.stateChanged.connect(self.update_t2_cursor)
+        self.t_cursor_widget.labels.stateChanged.connect(self.update_t_cursor_labels)
         self.t_cursor_widget.reset.clicked.connect(self.reset_t_cursors)
-        self.addAction(self.t_cursor_action)
+        action = QtWidgets.QWidgetAction(self)
+        action.setDefaultWidget(self.t_cursor_widget)
+        self.addAction(action)
 
+        row = self.addButtonRow()
+        button = QtWidgets.QPushButton("enable all on all plots")
+        button.clicked.connect(self.enable_all_t_cursors)
+        row.addWidget(button)
+        button = QtWidgets.QPushButton("disable all on all plots")
+        button.clicked.connect(self.disable_all_t_cursors)
+        row.addWidget(button)
+
+        self.addLabel()
         self.addSection("Y cursors")
+
         self.y_cursor_widget = CursorWidget('Y1', 'Y2')
-        self.y_cursor_action = QtWidgets.QWidgetAction(self)
-        self.y_cursor_action.setDefaultWidget(self.y_cursor_widget)
         self.y_cursor_widget.Y1.stateChanged.connect(self.update_y1_cursor)
         self.y_cursor_widget.Y2.stateChanged.connect(self.update_y2_cursor)
+        self.y_cursor_widget.labels.stateChanged.connect(self.update_y_cursor_labels)
         self.y_cursor_widget.reset.clicked.connect(self.reset_y_cursors)
-        self.addAction(self.y_cursor_action)
+        action = QtWidgets.QWidgetAction(self)
+        action.setDefaultWidget(self.y_cursor_widget)
+        self.addAction(action)
 
         self.addLabel()
         self.addSection("add/modify/remove channels")
-        self.addLabel()
-
-        self.openChannelSelectDialogButton = self.addButton("set channel list/parameters")
-        self.openChannelSelectDialogButton.clicked.connect(self.channel_select_dialog)
 
+        row = self.addButtonRow()
         self.addChannelEntry = QtWidgets.QLineEdit()
         self.addChannelEntry.setMinimumSize(300, 24)
-        self.addChannelEntry.setPlaceholderText("enter channel to add")
+        self.addChannelEntry.setPlaceholderText("enter channel to add to plot")
         self.addChannelEntry.setValidator(QtGui.QRegExpValidator(QtCore.QRegExp(CHANNEL_REGEXP)))
         self.addChannelEntry.textChanged.connect(self.validate_add)
         self.addChannelEntry.returnPressed.connect(self.add_channel)
         self.addChannelEntry.setAlignment(QtCore.Qt.AlignCenter | QtCore.Qt.AlignVCenter)
-        acea = QtWidgets.QWidgetAction(self)
-        acea.setDefaultWidget(self.addChannelEntry)
-        self.addAction(acea)
-
-        self.addChannelButton = self.addButton("add channel to plot")
+        row.addWidget(self.addChannelEntry)
+        self.addChannelButton = QtWidgets.QPushButton("add to plot")
         self.addChannelButton.setEnabled(False)
         self.addChannelButton.clicked.connect(self.add_channel)
+        row.addWidget(self.addChannelButton)
 
-        self.addLabel()
+        row = self.addButtonRow()
+        button = QtWidgets.QPushButton("configure channels for plot")
+        button.clicked.connect(self.channel_config_dialog)
+        row.addWidget(button)
 
+        row = self.addButtonRow()
         self.removeChannelList = QtWidgets.QComboBox()
         self.removeChannelList.setMinimumSize(200, 26)
         self.removeChannelList.currentIndexChanged.connect(self.remove_channel)
         # self.removeChannelList.setAlignment(QtCore.Qt.AlignCenter | QtCore.Qt.AlignVCenter)
-        rcl = QtWidgets.QWidgetAction(self)
-        rcl.setDefaultWidget(self.removeChannelList)
-        self.addAction(rcl)
+        row.addWidget(self.removeChannelList)
 
         self.addLabel()
         self.addSection("add/remove plots")
-        self.addLabel()
 
-        self.newPlotRowButton = self.addButton("add plot to row")
-        self.newPlotRowButton.clicked.connect(self.new_plot_row)
+        row = self.addButtonRow()
+        button = QtWidgets.QPushButton("add plot to column")
+        button.clicked.connect(self.new_plot_col)
+        row.addWidget(button)
+        button = QtWidgets.QPushButton("add plot to row")
+        button.clicked.connect(self.new_plot_row)
+        row.addWidget(button)
 
-        self.newPlotColButton = self.addButton("add plot to column")
-        self.newPlotColButton.clicked.connect(self.new_plot_col)
-
-        self.addLabel()
-
-        self.removePlotButton = self.addButton("remove plot")
-        self.removePlotButton.clicked.connect(self.remove_plot)
+        row = self.addButtonRow()
+        button = QtWidgets.QPushButton("remove plot")
+        button.clicked.connect(self.remove_plot)
+        row.addWidget(button)
 
         self.setContentsMargins(10, 10, 10, 10)
 
@@ -183,6 +199,14 @@ class NDScopePlotMenu(QtWidgets.QMenu):
 
     ##########
 
+    def set_title(self, pos=None):
+        loc = self.plot().loc
+        title = f"plot {loc}"
+        if pos:
+            title += f" @ ({pos.x():g}, {pos.y():g})"
+        self.setTitle(title)
+        self.titleLabel.setText(title)
+
     def addLabel(self, label=''):
         ql = QtWidgets.QLabel()
         ql.setText(label)
@@ -190,13 +214,20 @@ class NDScopePlotMenu(QtWidgets.QMenu):
         qla = QtWidgets.QWidgetAction(self)
         qla.setDefaultWidget(ql)
         self.addAction(qla)
+        return ql
 
-    def addButton(self, label):
-        button = QtWidgets.QPushButton(label)
+    def addButtonRow(self):
+        widget = QtWidgets.QWidget()
+        layout = QtWidgets.QHBoxLayout()
+        layout.setContentsMargins(0, 5, 0, 0)
+        widget.setLayout(layout)
         action = QtWidgets.QWidgetAction(self)
-        action.setDefaultWidget(button)
+        action.setDefaultWidget(widget)
+        # FIXME: this doesn't actually set the individual buttons to
+        # be "actions" such that the close the context menu when
+        # clicked
         self.addAction(action)
-        return button
+        return layout
 
     ##########
 
@@ -223,26 +254,43 @@ class NDScopePlotMenu(QtWidgets.QMenu):
                     self.yAxisUI.autoPercentSpin.setValue(int(state['autoRange'][i]*100))
             else:
                 self.yAxisUI.manualRadio.setChecked(True)
-            self.yAxisUI.logModeCheck.setChecked(state['logMode'][i])
+            # logMode state not present before PyQTGraph version 0.12
+            if 'logMode' in state:
+                self.yAxisUI.logModeCheck.setChecked(state['logMode'][i])
 
         if state['mouseMode'] == self.view().PanMode:
             self.mouseModeUI.pan.setChecked(True)
         else:
             self.mouseModeUI.rect.setChecked(True)
 
-        self.t_cursor_widget.T1.setChecked(self.plot().t_cursors.T1.isVisible())
-        self.t_cursor_widget.T2.setChecked(self.plot().t_cursors.T2.isVisible())
+        self.t_cursor_widget.T1.setChecked(self.plot().t_cursors.C1.isVisible())
+        self.t_cursor_widget.T2.setChecked(self.plot().t_cursors.C2.isVisible())
+        self.t_cursor_widget.labels.setChecked(self.plot().t_cursors.labels_are_visible())
 
-        self.y_cursor_widget.Y1.setChecked(self.plot().y_cursors.Y1.isVisible())
-        self.y_cursor_widget.Y2.setChecked(self.plot().y_cursors.Y2.isVisible())
+        self.y_cursor_widget.Y1.setChecked(self.plot().y_cursors.C1.isVisible())
+        self.y_cursor_widget.Y2.setChecked(self.plot().y_cursors.C2.isVisible())
+        self.y_cursor_widget.labels.setChecked(self.plot().y_cursors.labels_are_visible())
 
-    def popup(self, pos):
+    # HACK: QMenu popups are usually passed a global position.  for
+    # this ViewBox menu we have reimplemented the
+    # ViewBox.raiseContextMenu() method to send the MouseClickEvent
+    # instead, so that we can extract both the screen and scene
+    # positions.  The screen position is passed to the underlying
+    # QMenu.popup(), while the scene position is mapped to the view
+    # and cached so that it can be sent up to the Scope for certain
+    # actions (e.g. reset t0)
+    def popup(self, ev):
         self.updateState()
 
-        plot = self.plot()
+        pos = ev.screenPos().toPoint()
+        self.view_pos = self.view().mapSceneToView(ev.scenePos().toPoint())
 
-        ppos = plot.vb.mapSceneToView(pos)
-        self.pos = (ppos.x(), ppos.y())
+        self.set_title(self.view_pos)
+
+        if self.plot().data.online:
+            self.resetT0.setEnabled(False)
+        else:
+            self.resetT0.setEnabled(True)
 
         # update remove channels list
         self.update_channel_list()
@@ -257,9 +305,9 @@ class NDScopePlotMenu(QtWidgets.QMenu):
         else:
             self.addChannelEntry.setText('')
 
-        self.removeChannelList.setEnabled(len(plot.channels) > 0)
+        self.removeChannelList.setEnabled(len(self.plot().channels) > 0)
 
-        QtWidgets.QMenu.popup(self, pos)
+        super().popup(pos)
 
     ##########
 
@@ -268,7 +316,7 @@ class NDScopePlotMenu(QtWidgets.QMenu):
         self.view().autoRange()
 
     def reset_t0(self):
-        self.plot()._reset_t0(self.pos[0])
+        self.plot()._reset_t0(self.view_pos.x())
 
     ##########
 
@@ -294,20 +342,26 @@ class NDScopePlotMenu(QtWidgets.QMenu):
             self.addChannelEntry.setStyleSheet('')
             self.addChannelButton.setEnabled(False)
 
-    def channel_select_dialog(self):
-        self.plot()._select_channel_menu()
+    def channel_config_dialog(self):
+        self.plot().open_channel_config_dialog()
         self.close()
 
-    def add_channel(self):
+    def get_channel_from_entry(self):
         channel = str(self.addChannelEntry.text())
         if CHANNEL_RE.match(channel):
-            self.plot()._add_channel_menu(channel)
+            return channel
+        return None
+
+    def add_channel(self):
+        channel = self.get_channel_from_entry()
+        if channel:
+            self.plot().add_channels({channel: None})
         self.close()
 
     def remove_channel(self, *args):
         self.removeChannelList.currentIndexChanged.disconnect(self.remove_channel)
         channel = str(self.removeChannelList.currentText())
-        self.plot().remove_channel(channel)
+        self.plot().remove_channels([channel])
         self.removeChannelList.currentIndexChanged.connect(self.remove_channel)
         self.close()
 
@@ -318,12 +372,10 @@ class NDScopePlotMenu(QtWidgets.QMenu):
         self.new_plot('col')
 
     def new_plot(self, rowcol):
-        channel = str(self.addChannelEntry.text())
-        kwargs = {}
-        if CHANNEL_RE.match(channel):
-            kwargs['channels'] = [{channel: None}]
+        channel = self.get_channel_from_entry()
         self.plot().new_plot_request.emit(
-            (self.plot(), rowcol, kwargs),
+            (self.plot(), rowcol,
+             {'channels': [{channel: None}]}),
         )
         self.close()
 
@@ -373,26 +425,46 @@ class NDScopePlotMenu(QtWidgets.QMenu):
 
     def update_t1_cursor(self):
         self.plot().enable_t_cursors().set_visible(
-            t1=self.t_cursor_widget.T1.isChecked(),
+            C1=self.t_cursor_widget.T1.isChecked(),
         )
 
     def update_t2_cursor(self):
         self.plot().enable_t_cursors().set_visible(
-            t2=self.t_cursor_widget.T2.isChecked(),
+            C2=self.t_cursor_widget.T2.isChecked(),
         )
 
+    def update_t_cursor_labels(self):
+        self.plot().t_cursors.set_labels_visible(
+            self.t_cursor_widget.labels.isChecked(),
+        )
+
+    def enable_all_t_cursors(self):
+        self.plot().t_cursors_enable.emit(True)
+        self.close()
+
+    def disable_all_t_cursors(self):
+        self.plot().t_cursors_enable.emit(False)
+        self.close()
+
     def reset_t_cursors(self):
         self.plot().t_cursors.reset()
+        self.close()
 
     def update_y1_cursor(self):
         self.plot().enable_y_cursors().set_visible(
-            y1=self.y_cursor_widget.Y1.isChecked(),
+            C1=self.y_cursor_widget.Y1.isChecked(),
         )
 
     def update_y2_cursor(self):
         self.plot().enable_y_cursors().set_visible(
-            y2=self.y_cursor_widget.Y2.isChecked(),
+            C2=self.y_cursor_widget.Y2.isChecked(),
+        )
+
+    def update_y_cursor_labels(self):
+        self.plot().y_cursors.set_labels_visible(
+            self.y_cursor_widget.labels.isChecked(),
         )
 
     def reset_y_cursors(self):
         self.plot().y_cursors.reset()
+        self.close()
