@@ -23,7 +23,7 @@ from . import __version__
 from ._qt import load_ui
 from . import const
 from . import util
-from .data import DataStore
+from .cache import DataCache
 from .plot import NDScopePlot
 from .trigger import Trigger
 from .crosshair import Crosshair
@@ -51,6 +51,9 @@ STATUS_STYLES = {
     "error": "background: rgba(255,0,0,255); color: white; font-weight: bold;",
 }
 
+URL_MAIN = 'https://git.ligo.org/cds/software/ndscope'
+URL_RELEASE = 'https://git.ligo.org/cds/software/ndscope/-/blob/master/RELEASE.md?ref_type=heads'
+URL_ISSUE = 'https://git.ligo.org/cds/software/ndscope/issues'
 
 TREND_OPTIONS = [
     'auto',
@@ -96,11 +99,11 @@ class NDScope(*load_ui('scope.ui')):
         # add a little space
         self.statusBar.addWidget(QtWidgets.QLabel())
         self.statusBar.addWidget(self.labelServer)
-        self.labelChannelCount = QtWidgets.QLabel("[?]")
+        self.labelChannelCount = QtWidgets.QLabel("[waiting for channels...]")
         self.labelChannelCount.setToolTip("channel count")
         # the '1' add stretch to push the next label to the right side
         self.statusBar.addWidget(self.labelChannelCount, 1)
-        labelHome = QtWidgets.QLabel(f"<a href=https://git.ligo.org/cds/software/ndscope>ndscope {__version__}</a> (<a href=https://git.ligo.org/cds/software/ndscope/issues>report bug</a>)")
+        labelHome = QtWidgets.QLabel(f"<a href={URL_MAIN}>ndscope</a> <a href={URL_RELEASE}>{__version__}</a> (<a href={URL_ISSUE}>report bug</a>)")
         labelHome.setOpenExternalLinks(True)
         self.statusBar.addWidget(labelHome)
         self.statusClearButton = QtWidgets.QPushButton("clear")
@@ -113,11 +116,18 @@ class NDScope(*load_ui('scope.ui')):
         self.t0 = 0
         self._error = False
         self._dialogs = {}
-        self._channel_tree_model = None
-        self._channel_table_model = None
-        self._channel_select_dialog = None
-        self._channel_list_window = None
+        self._channel_list_widget = None
+        self._channel_config_dialog = channel_select.PlotChannelConfigDialog(self)
+        self._channel_config_dialog.channel_list_button.setEnabled(False)
+        self.channelListButton.setEnabled(False)
+        self.channelListButton2.setEnabled(False)
+
+        # this holds a reference to the channel select menu creation
+        # thread, so it's not inadvertently cleaned up
         self._bcsmt = None
+        # flag to indicate that we are in a base state with no
+        # channels declared
+        self._base = True
 
         # FIXME: HACK: this is an attempt to bypass the following bug:
         #
@@ -144,13 +154,12 @@ class NDScope(*load_ui('scope.ui')):
         ##########
         # data and data retrieval
 
-        self.data = DataStore()
+        self.data = DataCache()
         self.last_cmd = (None, {})
         self.last_data = (None, 0, 0)
 
         self.data.signal_channel_list_ready.connect(self._channel_list_ready_callback)
-        self.data.signal_channel_add.connect(self._channel_add_callback)
-        self.data.signal_channel_remove.connect(self._channel_remove_callback)
+        self.data.signal_channel_change.connect(self._channel_change_callback)
 
         self.data.signal_data_online_start.connect(self._data_online_start)
         self.data.signal_data_retrieve_start.connect(self._data_retrieve_start)
@@ -253,8 +262,9 @@ class NDScope(*load_ui('scope.ui')):
         # style
 
         self.fontSizeSelect.valueChanged.connect(self.set_font_size)
-        self.colorModeButtonGroup.buttonClicked.connect(self.set_color_mode)
         self.lineWidthSelect.valueChanged.connect(self.set_line_width)
+        self.gridAlphaSelect.valueChanged.connect(self.set_grid_alpha)
+        self.colorModeButtonGroup.buttonClicked.connect(self.set_color_mode)
 
         ##########
         # export
@@ -283,6 +293,9 @@ class NDScope(*load_ui('scope.ui')):
         self.resetRangeButton.clicked.connect(self.reset)
         self.resetRangeButton2.clicked.connect(self.reset)
 
+        self.channelListButton.clicked.connect(self.show_channel_list)
+        self.channelListButton2.clicked.connect(self.show_channel_list)
+
         ##########
         # button icons
         # FIXME: should set this in .ui somehow
@@ -312,7 +325,7 @@ class NDScope(*load_ui('scope.ui')):
         self.set_font_size(10)
         self.add_plot()
         self._set_t0(util.gpstime_parse('now').gps())
-        self.set_window(-2, 0)
+        self.set_online_window(2)
         self.last_cmd = ('start', {'window': self.get_window()})
 
     ##########
@@ -389,6 +402,13 @@ class NDScope(*load_ui('scope.ui')):
             for cc in plot.channels.values():
                 cc.set_params(width=width)
 
+    def set_grid_alpha(self, value):
+        """set grid line alpha for all plots
+
+        """
+        for plot in self.plots:
+            plot.showGrid(alpha=value)
+
     ##########
 
     def _base_ui(self):
@@ -565,7 +585,7 @@ class NDScope(*load_ui('scope.ui')):
             if plot == p:
                 return loc
 
-    def add_plot(self, channels=None, **kwargs):
+    def add_plot(self, channels=None, title=None, **kwargs):
         """Add plot to the scope
 
         If provided `channels` should be a list of channel:property
@@ -581,17 +601,19 @@ class NDScope(*load_ui('scope.ui')):
             if loc == ploc:
                 raise ValueError(f"Plot already exists at location {ploc}.")
 
-        plot = NDScopePlot(self.data, loc=loc)
+        plot = NDScopePlot(self.data, title=title, loc=loc)
         plot.set_font(self.font())
         plot.set_color_mode(self._color_mode)
-        plot.channel_select_request.connect(self.select_channels)
-        plot.request_reset.connect(self.reset)
-        plot.new_plot_request.connect(self._add_plot_handler)
-        plot.remove_plot_request.connect(self.remove_plot)
+
+        # connect plot signals
         plot.t0_reset.connect(self.reset_t0_relative)
+        plot.t_cursors_enable.connect(self._t_cursors_enable)
         plot.t_cursor_moved.connect(self._update_t_cursor)
         plot.log_mode_toggled.connect(self.crosshair.redraw)
         plot.log_mode_toggled.connect(self.trigger.redraw)
+        plot.channel_config_request.connect(self.config_channels)
+        plot.new_plot_request.connect(self._add_plot_handler)
+        plot.remove_plot_request.connect(self.remove_plot)
 
         # filter out non-layout arguments
         log_scale = kwargs.pop('log-scale', False)
@@ -625,14 +647,18 @@ class NDScope(*load_ui('scope.ui')):
             plot.setXLink(self.plot0)
 
         if channels:
-            # each channel should be a {name: curve_params} dict
-            for chan in channels:
-                name, ckwargs = list(chan.items())[0]
-                ckwargs = ckwargs or {}
-                plot.add_channel(name, **ckwargs)
+            # FIXME: the plot.add_channels method takes a dictionary
+            # of channel:curve items, but the template returns a list
+            # of single element channel:curve dicts.  the template
+            # handling should be fixed to just return a single
+            # channel:curve dictionary, but i think it was implemented
+            # this way so that the template yaml looked cleaner
+            channels = {list(chan.keys())[0]: list(chan.values())[0] for chan in channels}
+            plot.add_channels(channels)
 
         # set Y scale first because it necessarily turns on autoscale
         if log_scale:
+            plot.vb.menu.yAxisUI.logModeCheck.setChecked(True)
             plot.set_log_mode(log_scale)
         if y_range:
             plot.set_y_range(y_range)
@@ -671,10 +697,7 @@ class NDScope(*load_ui('scope.ui')):
             if loc == (0, 0):
                 self.status_error("Can not remove (0,0) plot.")
                 return
-        # first remove all channels
-        # make copy of list cause we'll be changing it
-        for chan in list(plot.channels.keys()):
-            plot.remove_channel(chan)
+        plot.remove_channels()
         plot.log_mode_toggled.disconnect()
         self.plotLayout.removeItem(plot)
 
@@ -695,81 +718,68 @@ class NDScope(*load_ui('scope.ui')):
         self._bcsmt = t
 
     def _channel_models_ready_callback(self, recv):
-        self._channel_tree_model, self._channel_table_model = recv
+        channel_tree_model, channel_table_model = recv
         nchannels = len(self.data.available_channels)
-        channel_list_widget = channel_select.ChannelListWidget(self._channel_tree_model, self._channel_table_model)
-        channel_list_widget.title.setText(f"Channel list for <span style='font-weight: bold'>{self.server_formatted}</span> [{nchannels} channels]")
-        self._channel_select_dialog = channel_select.ChannelSelectDialog(channel_list_widget, self)
+        self._channel_list_widget = channel_select.ChannelListWidget(
+            channel_tree_model, channel_table_model)
+        self._channel_list_widget.set_server_info(self.server_formatted, nchannels)
+        self._channel_list_dialog = channel_select.ChannelListDialog(
+            self._channel_list_widget)
+        self._channel_config_dialog.channel_list_button.clicked.connect(self.show_channel_list)
+        self.channelListButton.setEnabled(True)
+        self.channelListButton2.setEnabled(True)
+        self._channel_config_dialog.channel_list_button.setEnabled(True)
         self._bcsmt = None
-        logger.debug("channel select dialog ready.")
+        logger.debug("channel list ready.")
         self.labelChannelCount.setText(f"[{nchannels} channels]")
         self._channel_models_ready.emit()
 
-    def select_channels(self, plot=None):
-        """Present the channel select dialog
+    def show_channel_list(self):
+        """Show the channel list window
 
-        Will block the scope to wait for the channel list if it's not
-        available yet.
+        """
+        self._channel_list_dialog.show()
+
+    def config_channels(self, plot=None):
+        """Present the channel select dialog
 
         If no plot is specified, assume plot0
 
         """
-        if 'select' in self._dialogs and self._dialogs['select'].isVisible():
+        if 'config' in self._dialogs and self._dialogs['config'].isVisible():
             return
         if not plot:
             plot = self.plot0
-        if self._channel_select_dialog:
-            self._channel_select_present(plot)
-        else:
-            if not self._bcsmt:
-                self.data.fetch_channel_list_async()
-            self._channel_select_wait(plot)
-
-    def _channel_select_wait(self, plot):
-        d = QtWidgets.QLabel("waiting for channel list...", self)
-        d.setMargin(10)
-        d.setSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
-        self._dialog_create('channel-wait', d)
-        self._channel_models_ready.connect(lambda: self._channel_select_wait_done(plot))
-
-    def _channel_select_wait_done(self, plot):
-        self._dialog_cleanup('channel-wait')
-        self._channel_select_present(plot)
-
-    def _channel_select_present(self, plot):
-        self._channel_select_dialog.setSelectedChannelList(plot.channels)
-        self._channel_select_dialog.setTitlePlot(self.get_plot_location(plot))
+        self._channel_config_dialog.set_plot(plot)
         self._dialog_create(
-            'select', self._channel_select_dialog,
-            callback=lambda chans: self._channel_select_dialog_result(plot, chans),
+            'config', self._channel_config_dialog,
+            callback=lambda chans: self._channel_config_dialog_result(plot, chans),
         )
 
-    # SLOT
-    def _channel_select_dialog_result(self, plot, channel_list):
-        self._dialog_cleanup('select')
+    def _channel_config_dialog_result(self, plot, channel_list):
+        self._dialog_cleanup('config')
+        if channel_list is None:
+            return
         plot.set_channel_objs(channel_list)
-        self.reset()
 
-    def _channel_add_callback(self, recv):
-        """callback for when channel added"""
-        channel, error = recv
+    def _channel_change_callback(self, recv):
+        """callback to update scope when channel added or removed"""
+        error = recv
         if error:
             self.status_error(error)
             return
-        logger.info(f"added channel: {channel}")
-        self._update_triggerSelect()
-
-    def _channel_remove_callback(self, recv):
-        """callback for when channel removed from plot"""
-        channel, error = recv
-        if error:
-            self.status_error(error)
-            return
-        self.status_clear()
-        logger.info(f"removed channel: {channel}")
         self._update_triggerSelect()
         if self.data.empty:
+            self._base = True
             self._base_ui()
+            self.select_channels()
+        elif self._base:
+            # if we were in the base state (no channels) start the
+            # online stream when new channels are added
+            self._base = False
+            self.start()
+        else:
+            self._offline_ui()
 
     def load_template(self, template):
         """Load scope template from template dictionary
@@ -777,6 +787,10 @@ class NDScope(*load_ui('scope.ui')):
         """
         logger.info(f"loading template...")
 
+        # remove the base state flag immediately, so that we don't
+        # start online streams during template loading
+        self._base = False
+
         # If there is more than one plot then check the layout to make sure
         # there's a (0,0) plot specified.
         # If there is only one plot then move it to (0,0).
@@ -855,6 +869,10 @@ class NDScope(*load_ui('scope.ui')):
         if line_width:
             self.lineWidthSelect.setValue(line_width)
 
+        grid_alpha = template.get('grid-alpha')
+        if grid_alpha:
+            self.gridAlphaSelect.setValue(grid_alpha)
+
         t_cursors = template.get('t-cursors')
         if t_cursors:
             self.load_t_cursors(t_cursors)
@@ -863,6 +881,10 @@ class NDScope(*load_ui('scope.ui')):
         if time_axis_mode:
             self.set_time_axis_mode(time_axis_mode)
 
+        if not self.data.channels:
+            self._base = True
+            return
+
         t0 = template.get('t0')
         window = template.get('time-window')
         logger.info(f"t0={t0}, window={window}")
@@ -872,9 +894,10 @@ class NDScope(*load_ui('scope.ui')):
             self.start(window)
 
     ##########
-    # RANGE AND SPAN
+    # WINDOW/RANGE/SPAN
 
     def get_window(self):
+        """tuple of (xmin, xmax) values"""
         (xmin, xmax), (ymin, ymax) = self.plot0.viewRange()
         return xmin, xmax
 
@@ -890,6 +913,24 @@ class NDScope(*load_ui('scope.ui')):
         start, end = self.get_range()
         return abs(end - start)
 
+    def _set_window(self, xmin, xmax):
+        logger.debug("set window: {} {}".format(xmin, xmax))
+        self.plot0.setXRange(xmin, xmax, padding=0, update=False)
+
+    def set_window(self, xmin, xmax):
+        """set xmin/xmax values"""
+        self._set_window(xmin, xmax)
+        self.update_entryWindow()
+
+    def set_online_window(self, span=None):
+        """set the window for online, e.g. (-abs(span), 0)
+
+        If span not specified use current span.
+        """
+        if not span:
+            span = self.get_span()
+        self.set_window(-abs(span), const.ONLINE_X_MAX)
+
     def preferred_trend(self):
         """preferred trend for the current time span"""
         trend = TREND_OPTIONS[self.trendSelectGroup.checkedId()]
@@ -917,8 +958,7 @@ class NDScope(*load_ui('scope.ui')):
             span = abs(min(window))
         else:
             span = self.get_span()
-            window = (-span, 0)
-        self.set_window(*window)
+        self.set_online_window(span)
         trend = self.preferred_trend()
         # Sets the visibility of the curves in case the trend type has changed.
         # See the comment in `update_all_curves_visibility` for the reason this is needed.
@@ -1055,6 +1095,12 @@ class NDScope(*load_ui('scope.ui')):
         self._disable_crosshair()
         self._set_time_axis_mode('relative')
 
+    # SLOT
+    def _data_online_done(self, signal):
+        if self.crosshairGroup.isChecked():
+            self._enable_crosshair()
+        self._reset_time_axis_mode()
+
     # SLOT
     def _data_retrieve_start(self, msg):
         self.status_message(msg, style="data", log=False)
@@ -1064,12 +1110,6 @@ class NDScope(*load_ui('scope.ui')):
         self.fetchButton1.setEnabled(False)
         self.fetchButton2.setEnabled(False)
 
-    # SLOT
-    def _data_online_done(self, signal):
-        if self.crosshairGroup.isChecked():
-            self._enable_crosshair()
-        self._reset_time_axis_mode()
-
     # SLOT
     def _data_retrieve_done(self, signal):
         if self.data.empty:
@@ -1181,6 +1221,7 @@ class NDScope(*load_ui('scope.ui')):
         xd = (end-start)/2
         self.fetch(t0=t0, window=(-xd, xd))
 
+    # SLOT used when reseting t0
     def reset_t0_relative(self, val):
         if self.data.online:
             return
@@ -1196,14 +1237,6 @@ class NDScope(*load_ui('scope.ui')):
             t2 -= val
             self.set_t_cursor_values(t1, t2)
 
-    def _set_window(self, xmin, xmax):
-        logger.debug("set window: {} {}".format(xmin, xmax))
-        self.plot0.setXRange(xmin, xmax, padding=0, update=False)
-
-    def set_window(self, xmin, xmax):
-        self._set_window(xmin, xmax)
-        self.update_entryWindow()
-
     ##########
     # TIMES
 
@@ -1394,6 +1427,7 @@ class NDScope(*load_ui('scope.ui')):
             tplot = None
         if channel is not None:
             nplot = self.plots4chan(channel)[0]
+            self.trigger.set_color(nplot.channels[channel].get_QColor())
         else:
             nplot = None
         self.trigger.channel = channel
@@ -1404,16 +1438,12 @@ class NDScope(*load_ui('scope.ui')):
                 nplot.addItem(self.trigger.line, ignoreBounds=True)
                 nplot.disableAutoRange(axis='y')
             self.trigger.plot = nplot
-            self.trigger.redraw()
-            return True
-        else:
-            return False
+        self.reset_trigger_level()
 
     def _update_trigger_channel(self):
         """update trigger channel from menu select"""
         chan = str(self.triggerSelect.currentText())
-        if self.set_trigger_channel(chan):
-            self.reset_trigger_level()
+        self.set_trigger_channel(chan)
         logger.info("trigger set: {}".format(chan))
 
     def _update_triggerSelect(self):
@@ -1470,8 +1500,7 @@ class NDScope(*load_ui('scope.ui')):
     def _disable_trigger(self):
         """disable trigger, base method"""
         self.set_trigger_channel(None)
-        # span = self.get_span()
-        # self.set_window(-span, 0)
+        self.set_online_window()
         logger.info("trigger disabled")
 
     def disable_trigger(self):
@@ -1576,10 +1605,15 @@ class NDScope(*load_ui('scope.ui')):
     ##########
     # CURSORS
 
+    def _t_cursors_enable(self, val):
+        assert isinstance(val, bool), val
+        for plot in self.plots:
+            plot.enable_t_cursors().set_visible(C1=val, C2=val)
+
     def _update_t_cursor(self, indval):
         index, value = indval
         for plot in self.plots:
-            plot.t_cursors.set_values(**{index.lower(): value})
+            plot.t_cursors.set_values(**{index: value})
 
     def get_t_cursor_values(self):
         return self.plot0.t_cursors.get_values()
@@ -1612,6 +1646,7 @@ class NDScope(*load_ui('scope.ui')):
             'color-mode': self._color_mode,
             'window-title': self.windowTitle().replace('ndscope: ', ''),
             'font-size': self.font().pointSize(),
+            'grid-alpha': self.gridAlphaSelect.value(),
         }
         plots = []
         for plot, cells in self.plotLayout.items.items():
@@ -1626,7 +1661,7 @@ class NDScope(*load_ui('scope.ui')):
                 plot_item['y-range'] = 'auto'
             else:
                 # convert to float from numpy.float64
-                plot_item['y-range'] = list(map(float, plot.vb.viewRange()[1]))
+                plot_item['y-range'] = [plot.y_pos_to_val(v) for v in (map(float, plot.vb.viewRange()[1]))]
             if plot.log_mode:
                 plot_item['log-scale'] = True
             if plot.t_cursors.are_visible():
