@@ -4,14 +4,14 @@ from .. import template
 
 
 def test_load_template():
-    template.COLOR_INDEX = 0
+    template._CHANNEL_COLORS = {}
     path = resource_filename(__name__, 'templates/test.yaml')
     t, _ = template.load_template(path)
     assert t['time-window'] == -100
 
 
 def test_create_template_chans0():
-    template.COLOR_INDEX = 0
+    template._CHANNEL_COLORS = {}
     chans = ['A', 'B', 'C']
     t0, l0 = template.template_from_chans(chans)
     template.convert_layout(t0, 'grid')
@@ -21,7 +21,7 @@ def test_create_template_chans0():
 
 
 def test_create_template_chans1():
-    template.COLOR_INDEX = 0
+    template._CHANNEL_COLORS = {}
     chans = ['A', '.', 'B', 'C', '.', 'D']
     t0, l0 = template.template_from_chans(chans)
     path = resource_filename(__name__, 'templates/test1.yaml')
@@ -30,7 +30,7 @@ def test_create_template_chans1():
 
 
 def test_template_convert_grid():
-    template.COLOR_INDEX = 0
+    template._CHANNEL_COLORS = {}
     path = resource_filename(__name__, 'templates/test1.yaml')
     t0, l0 = template.load_template(path)
     template.convert_layout(t0, 'grid')
@@ -40,7 +40,7 @@ def test_template_convert_grid():
 
 
 def test_template_convert_stack():
-    template.COLOR_INDEX = 0
+    template._CHANNEL_COLORS = {}
     path = resource_filename(__name__, 'templates/test1.yaml')
     t0, l0 = template.load_template(path)
     template.convert_layout(t0, 'stack')
@@ -50,7 +50,7 @@ def test_template_convert_stack():
 
 
 def test_template_convert_single():
-    template.COLOR_INDEX = 0
+    template._CHANNEL_COLORS = {}
     path = resource_filename(__name__, 'templates/test1.yaml')
     t0, l0 = template.load_template(path)
     template.convert_layout(t0, 'single')
@@ -70,7 +70,7 @@ def test_load_template_stp():
 
 
 def test_load_template_txt():
-    template.COLOR_INDEX = 0
+    template._CHANNEL_COLORS = {}
     path = resource_filename(__name__, 'templates/test3.txt')
     t0, l0 = template.load_template(path)
     assert l0 == 'grid'
@@ -81,7 +81,7 @@ def test_load_template_txt():
 
 
 def test_load_template_xml():
-    template.COLOR_INDEX = 0
+    template._CHANNEL_COLORS = {}
     path = resource_filename(__name__, 'templates/test4.xml')
     t0, l0 = template.load_template(path)
     assert l0 == 'grid'
