@@ -4,7 +4,7 @@ from qtpy import QtCore
 from qtpy.QtCore import Signal
 import pyqtgraph as pg
 
-from .const import LABEL_FILL, COLOR_MODE
+from .const import COLOR_MODE, LABEL_ALPHA
 
 
 class Trigger(QtCore.QObject):
@@ -21,11 +21,15 @@ class Trigger(QtCore.QObject):
         self.line = pg.InfiniteLine(
             angle=0,
             movable=True,
+            pen= {
+                'style': QtCore.Qt.DotLine,
+                'width': 3,
+            },
             label='trigger level',
             labelOpts={
                 'position': 0,
                 'anchors': [(0, 0.5), (0, 0.5)],
-                'fill': LABEL_FILL,
+                'fill': (0, 0, 0, LABEL_ALPHA),
             },
         )
         self.line.sigPositionChanged.connect(self._update_level_from_line)
@@ -41,10 +45,16 @@ class Trigger(QtCore.QObject):
         """set color mode"""
         fg = COLOR_MODE[mode]['fg']
         bg = COLOR_MODE[mode]['bg']
-        self.line.label.fill = bg
+        fill_color = bg.color()
+        fill_color.setAlpha(LABEL_ALPHA)
+        self.line.label.fill.setColor(fill_color)
         self.line.label.setColor(fg)
         self.line.pen.setColor(fg)
 
+    def set_color(self, color):
+        """set the trigger line pen color"""
+        self.line.pen.setColor(color)
+
     @property
     def active(self):
         """True if trigger is active"""
