@@ -1,3 +1,4 @@
+import os
 from collections import namedtuple
 
 from gpstime import gpstime, GPSTimeException
@@ -5,6 +6,28 @@ from gpstime import gpstime, GPSTimeException
 from . import const
 
 
+def resolve_ndsserver(ndsserver=None):
+    if not ndsserver:
+        ndsserver = os.getenv('NDSSERVER', const.NDSSERVER).split(',')[0]
+    ndsserver = ndsserver.lower()
+    return const.NDSSERVER_ALIAS_MAP.get(ndsserver, ndsserver)
+
+
+def format_nds_server_string():
+    server = os.getenv('NDSSERVER')
+    hostport = server.split(':')
+    try:
+        host, port = hostport
+    except ValueError:
+        host = hostport[0]
+        port = 31200
+    formatted = f"{host}"
+    #formatted = f'<span style="color:blue">{host}</span>'
+    if int(port) != 31200:
+        formatted += f":{port}"
+    return server, formatted
+
+
 def gpstime_parse(time):
     if time is None:
         return None
@@ -66,15 +89,15 @@ class TDStr:
         years, days = divmod(days, 365)
         # weeks, days = divmod(days, 7)
         self.td = TDUnits(
-            years,
-            # weeks,
-            days,
-            hours,
-            minutes,
-            seconds,
-            msecs,
-            usecs,
-            # nsecs,
+            int(years),
+            # int(weeks),
+            int(days),
+            int(hours),
+            int(minutes),
+            int(seconds),
+            int(msecs),
+            int(usecs),
+            # int(nsecs),
         )
 
     def __getitem__(self, item):
